SET @OLD_UNIQUE_CHECKS=@@UNIQUE_CHECKS, UNIQUE_CHECKS=0;
SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0;
SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='TRADITIONAL';

DROP SCHEMA IF EXISTS `newsAnalyzer` ;
CREATE SCHEMA IF NOT EXISTS `newsAnalyzer` DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci ;
USE `newsAnalyzer` ;

-- -----------------------------------------------------
-- Table `newsAnalyzer`.`ArticleSource`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `newsAnalyzer`.`ArticleSource` ;

CREATE  TABLE IF NOT EXISTS `newsAnalyzer`.`ArticleSource` (
  `articleSourceId` INT UNSIGNED NOT NULL AUTO_INCREMENT ,
  `articleSourceName` VARCHAR(200) NOT NULL ,
  PRIMARY KEY (`articleSourceId`) ,
  UNIQUE INDEX `unique` (`articleSourceName` ASC) )
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `newsAnalyzer`.`Article`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `newsAnalyzer`.`Article` ;

CREATE  TABLE IF NOT EXISTS `newsAnalyzer`.`Article` (
  `articleId` INT UNSIGNED NOT NULL AUTO_INCREMENT ,
  `articleTitle` VARCHAR(500) NOT NULL ,
  `articleDate` DATE NOT NULL ,
  `articleUrl` VARCHAR(200) NULL ,
  `articleSourceId` INT UNSIGNED NOT NULL ,
  `analyzed` INT UNSIGNED NOT NULL DEFAULT 0 ,
  PRIMARY KEY (`articleId`) ,
  INDEX `fk_Article_Font1` (`articleSourceId` ASC) ,
  CONSTRAINT `fk_Article_Font1`
    FOREIGN KEY (`articleSourceId` )
    REFERENCES `newsAnalyzer`.`ArticleSource` (`articleSourceId` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `newsAnalyzer`.`Facet`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `newsAnalyzer`.`Facet` ;

CREATE  TABLE IF NOT EXISTS `newsAnalyzer`.`Facet` (
  `facetId` INT UNSIGNED NOT NULL ,
  `facetName` VARCHAR(100) NOT NULL ,
  PRIMARY KEY (`facetId`) ,
  UNIQUE INDEX `facetName_UNIQUE` (`facetName` ASC) )
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `newsAnalyzer`.`NormalizedTag`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `newsAnalyzer`.`NormalizedTag` ;

CREATE  TABLE IF NOT EXISTS `newsAnalyzer`.`NormalizedTag` (
  `normalizedTagId` INT UNSIGNED NOT NULL AUTO_INCREMENT ,
  `normalizedTagName` VARCHAR(250) NOT NULL ,
  `normalizedTagFacetId` INT UNSIGNED NOT NULL ,
  PRIMARY KEY (`normalizedTagId`) ,
  UNIQUE INDEX `UNIQUE` (`normalizedTagName` ASC, `normalizedTagFacetId` ASC) ,
  INDEX `fk_NormalizedTag_Facet1` (`normalizedTagFacetId` ASC) ,
  CONSTRAINT `fk_NormalizedTag_Facet1`
    FOREIGN KEY (`normalizedTagFacetId` )
    REFERENCES `newsAnalyzer`.`Facet` (`facetId` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `newsAnalyzer`.`TagDraft`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `newsAnalyzer`.`TagDraft` ;

CREATE  TABLE IF NOT EXISTS `newsAnalyzer`.`TagDraft` (
  `tagDraftId` INT UNSIGNED NOT NULL AUTO_INCREMENT ,
  `tagDraftName` VARCHAR(250) NOT NULL ,
  `tagDraftCathegoryId` INT UNSIGNED NULL ,
  `tagDraftFacetId` INT UNSIGNED NULL ,
  `normalizedTagId` INT UNSIGNED NULL ,
  PRIMARY KEY (`tagDraftId`) ,
  UNIQUE INDEX `UNIQUE` (`tagDraftName` ASC, `tagDraftCathegoryId` ASC, `tagDraftFacetId` ASC) ,
  INDEX `fk_TagDraft_NormalizedTag1` (`normalizedTagId` ASC) ,
  INDEX `fk_TagDraft_Facet1` (`tagDraftFacetId` ASC) ,
  CONSTRAINT `fk_TagDraft_NormalizedTag1`
    FOREIGN KEY (`normalizedTagId` )
    REFERENCES `newsAnalyzer`.`NormalizedTag` (`normalizedTagId` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_TagDraft_Facet1`
    FOREIGN KEY (`tagDraftFacetId` )
    REFERENCES `newsAnalyzer`.`Facet` (`facetId` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_TagDraft_TagDraft1`
    FOREIGN KEY (`tagDraftCathegoryId` )
    REFERENCES `newsAnalyzer`.`TagDraft` (`tagDraftId` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `newsAnalyzer`.`Author`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `newsAnalyzer`.`Author` ;

CREATE  TABLE IF NOT EXISTS `newsAnalyzer`.`Author` (
  `authorId` INT UNSIGNED NOT NULL AUTO_INCREMENT ,
  `authorName` VARCHAR(200) NOT NULL ,
  PRIMARY KEY (`authorId`) ,
  INDEX `unique` (`authorName` ASC) )
ENGINE = InnoDB
COMMENT = '\n' ;


-- -----------------------------------------------------
-- Table `newsAnalyzer`.`ArticleHasTagDraft`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `newsAnalyzer`.`ArticleHasTagDraft` ;

CREATE  TABLE IF NOT EXISTS `newsAnalyzer`.`ArticleHasTagDraft` (
  `articleId` INT UNSIGNED NOT NULL ,
  `tagDraftId` INT UNSIGNED NOT NULL ,
  PRIMARY KEY (`articleId`, `tagDraftId`) ,
  INDEX `fk_Article_has_personFacet_personFacet1` (`tagDraftId` ASC) ,
  INDEX `fk_Article_has_personFacet_Article1` (`articleId` ASC) ,
  CONSTRAINT `fk_Article_has_personFacet_Article1`
    FOREIGN KEY (`articleId` )
    REFERENCES `newsAnalyzer`.`Article` (`articleId` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_Article_has_personFacet_personFacet1`
    FOREIGN KEY (`tagDraftId` )
    REFERENCES `newsAnalyzer`.`TagDraft` (`tagDraftId` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `newsAnalyzer`.`ArticleHasAuthor`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `newsAnalyzer`.`ArticleHasAuthor` ;

CREATE  TABLE IF NOT EXISTS `newsAnalyzer`.`ArticleHasAuthor` (
  `articleId` INT UNSIGNED NOT NULL ,
  `authorId` INT UNSIGNED NOT NULL ,
  PRIMARY KEY (`articleId`, `authorId`) ,
  INDEX `fk_Article_has_Author_Author1` (`authorId` ASC) ,
  INDEX `fk_Article_has_Author_Article1` (`articleId` ASC) ,
  CONSTRAINT `fk_Article_has_Author_Article1`
    FOREIGN KEY (`articleId` )
    REFERENCES `newsAnalyzer`.`Article` (`articleId` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_Article_has_Author_Author1`
    FOREIGN KEY (`authorId` )
    REFERENCES `newsAnalyzer`.`Author` (`authorId` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `newsAnalyzer`.`ArticleHasNormalizedTag`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `newsAnalyzer`.`ArticleHasNormalizedTag` ;

CREATE  TABLE IF NOT EXISTS `newsAnalyzer`.`ArticleHasNormalizedTag` (
  `articleId` INT UNSIGNED NOT NULL ,
  `normalizedTagId` INT UNSIGNED NOT NULL ,
  PRIMARY KEY (`articleId`, `normalizedTagId`) ,
  INDEX `fk_Article_has_Tag_Article1` (`articleId` ASC) ,
  INDEX `fk_ArticleClassifiedAs_Classifier1` (`normalizedTagId` ASC) ,
  CONSTRAINT `fk_Article_has_Tag_Article1`
    FOREIGN KEY (`articleId` )
    REFERENCES `newsAnalyzer`.`Article` (`articleId` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_ArticleClassifiedAs_Classifier1`
    FOREIGN KEY (`normalizedTagId` )
    REFERENCES `newsAnalyzer`.`NormalizedTag` (`normalizedTagId` )
    ON DELETE NO ACTION
    ON UPDATE NO ACTION)
ENGINE = InnoDB;



SET SQL_MODE=@OLD_SQL_MODE;
SET FOREIGN_KEY_CHECKS=@OLD_FOREIGN_KEY_CHECKS;
SET UNIQUE_CHECKS=@OLD_UNIQUE_CHECKS;

-- -----------------------------------------------------
-- Data for table `newsAnalyzer`.`Facet`
-- -----------------------------------------------------
START TRANSACTION;
USE `newsAnalyzer`;
INSERT INTO `newsAnalyzer`.`Facet` (`facetId`, `facetName`) VALUES (0, 'NULL');
INSERT INTO `newsAnalyzer`.`Facet` (`facetId`, `facetName`) VALUES (1, 'DESCRIPTIVE_SUBJECT');
INSERT INTO `newsAnalyzer`.`Facet` (`facetId`, `facetName`) VALUES (2, 'PERSON_NAME');
INSERT INTO `newsAnalyzer`.`Facet` (`facetId`, `facetName`) VALUES (3, 'GEOGRAPHIC_LOCATION');
INSERT INTO `newsAnalyzer`.`Facet` (`facetId`, `facetName`) VALUES (4, 'ORGANIZATION_NAME');
INSERT INTO `newsAnalyzer`.`Facet` (`facetId`, `facetName`) VALUES (5, 'TIME_RELATED');

COMMIT;
