/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package newsAnalyzer.analyzer;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import newsAnalyzer.NewsAnalyzerProperties;
import newsAnalyzer.dataModel.Article;
import newsAnalyzer.dataModel.DAOs.DAOFactory;
import newsAnalyzer.dataModel.DAOs.searchCriteriaObjects.ArticleCriteria;
import newsAnalyzer.dataModel.DAOs.searchCriteriaObjects.SearchCriteria;
import newsAnalyzer.dataModel.NormalizedTag;
import newsAnalyzer.dataModel.TagDraft;
import newsAnalyzer.analyzer.utils.TagNormalizer;
import newsAnalyzer.utils.TextUtils;

/**
 *
 * @author rcastari
 */
public class AnalyzerModule extends Thread {

    HashMap<TagDraft,TagDraft> tagDraftMap = new HashMap<TagDraft,TagDraft>();
    HashMap<NormalizedTag,NormalizedTag> normalizedTagMap = new HashMap<NormalizedTag,NormalizedTag>();

    private boolean stop = false;
    private boolean end = false;
    private long updateTime;

    public AnalyzerModule(){
    }

    private void analyze(Article article, Connection conn) throws Exception {

        //Falta controlar que el normalizedTag no existeixi ja en la llista de nts de l'article

        HashMap<Integer,Integer> associatedNTs = new HashMap<Integer,Integer>();

        List<TagDraft> tagDrafts = article.getTagDraftsList();

        int tabs;

        for (int tdi = 0; tdi < tagDrafts.size(); tdi++) {

            tabs = 0;

            TagDraft td = tagDrafts.get(tdi);

            if (NewsAnalyzerProperties.getInstance().debug()) {
                System.out.println(TextUtils.insertBlanks(tabs) + "Tag draft to normalize: ");
                tabs++;
                System.out.println(td.toString(tabs));
            }

            TagDraft inMemoryTD = tagDraftMap.get(td);
            if (inMemoryTD != null && td != inMemoryTD) {
                if (NewsAnalyzerProperties.getInstance().debug()) {
                    System.out.println(TextUtils.insertBlanks(tabs)+"Tag draft in memmory!");
                }
                tagDrafts.set(tdi, inMemoryTD);
                td = inMemoryTD;
                td.setNormalizedTag(td.getNormalizedTag());
            } else {
                td = DAOFactory.getDAOFactory().getTagDraftDAO().load(td, conn);
                tagDraftMap.put(td, td);

                NormalizedTag nt = td.getNormalizedTag();

                if (nt == null) {

                    if (NewsAnalyzerProperties.getInstance().debug()) {
                        System.out.println(TextUtils.insertBlanks(tabs)+"Normalizing tag draft...");
                    }

                    //Normalizing Tag Draft

                    nt = TagNormalizer.normalize(td);

                    if (nt != null) {

                        NormalizedTag inMemmoryNT = normalizedTagMap.get(nt);
                        if (inMemmoryNT != null && inMemmoryNT != nt) {
                            nt = inMemmoryNT;
                        } else {
                            nt = DAOFactory.getDAOFactory().getNormalizedTagDAO().load(nt, conn);
                            normalizedTagMap.put(nt, nt);
                        }

                        td.setNormalizedTag(nt);
                        DAOFactory.getDAOFactory().getTagDraftDAO().save(td, conn);
                    }

                } else {
                    if (NewsAnalyzerProperties.getInstance().debug()) {
                        System.out.println(TextUtils.insertBlanks(tabs)+"Tag Draft previously normalized!");
                    }
                }

            }

            NormalizedTag nt = td.getNormalizedTag();

            if (nt != null) {
                if(associatedNTs.get(nt.getId())==null){
                    article.getNormalizedTags(nt.getFacet()).add(nt);
                    DAOFactory.getDAOFactory().getArticleDAO().associateNormalizedTag(article, nt, conn);
                    associatedNTs.put(nt.getId(), nt.getId());
                }
            }

            if (NewsAnalyzerProperties.getInstance().debug()) {
                System.out.print(TextUtils.insertBlanks(tabs)+"Normalized Tag: ");
                if (nt != null) {
                    System.out.print(System.getProperty("line.separator"));
                    System.out.println(nt.toString(tabs+1));
                } else {
                    System.out.println("{}");
                }
            }
        }

    }

    private void analyzeNotAnalyzedArticles() {

        Article ad = new Article();
        ad.setHasBeenAnalyzed(false);

        ArticleCriteria ac = (ArticleCriteria) SearchCriteria.getSearchCriteria(ad);
        ac.setNullDiscriminatorCriteria(ArticleCriteria.URL, false);

        List<Article> al = new ArrayList<Article>();

        try {
            al = DAOFactory.getDAOFactory().getArticleDAO().find(ac);
        } catch (SQLException ex) {
            Logger.getLogger(AnalyzerModule.class.getName()).log(Level.SEVERE, null, ex);
        }

        if (NewsAnalyzerProperties.getInstance().debug()) {

            System.out.println("");
            System.out.println("=========================================");
            System.out.println("=          NORMALIZATION                =");
            System.out.println("=========================================");
            System.out.println("");
        }


        Connection conn = DAOFactory.getDAOFactory().getConnection();

        Iterator<Article> ai = al.iterator();

        while (ai.hasNext() && !stop) {

            Article a = ai.next();
            try {
                analyze(a, conn);
            } catch (Exception ex) {
                Logger.getLogger(AnalyzerModule.class.getName()).log(Level.SEVERE, null, ex);
            }
            try {
                DAOFactory.getDAOFactory().getArticleDAO().setHasBeenAnalyzed(a, conn);
            } catch (SQLException ex) {
                Logger.getLogger(AnalyzerModule.class.getName()).log(Level.SEVERE, null, ex);
            }

            if (NewsAnalyzerProperties.getInstance().debug()) {
                System.out.println(a);
            }
        }

        if (conn != null) {
            try {
                conn.close();
            } catch (SQLException ex) {
                //Can't do anyhing
            }
        }
    }

    @Override
    public void run(){
        while(!end){
            while(!stop){
                this.analyzeNotAnalyzedArticles();
                this.invokeWait(updateTime);
            }
        }
        if(!end){
            this.invokeWait();
        }
    }

    private synchronized void invokeWait(){
        try {
            this.wait();
        } catch (InterruptedException ex) {
            Logger.getLogger(AnalyzerModule.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    private synchronized void invokeWait(long milliseconds){
        try {
            this.wait(milliseconds);
        } catch (InterruptedException ex) {
            Logger.getLogger(AnalyzerModule.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public synchronized void analyzeNotAnalyzedArticles(long milliseconds) {
        stop=false;
        if(!this.isAlive()){
            this.start();
        }else{
            this.notify();
        }
    }

    public synchronized void invokeStop(){
        stop = true;
        this.notify();
    }

    public synchronized void invokeEnd(){
        stop = true;
        end = true;
        this.notify();
    }

}
