/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package newsAnalyzer.analyzer.utils;

import com.sun.org.apache.xerces.internal.parsers.DOMParser;
import edu.smu.tspell.wordnet.NounSynset;
import edu.smu.tspell.wordnet.Synset;
import edu.smu.tspell.wordnet.WordNetDatabase;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.util.logging.Level;
import java.util.logging.Logger;
import newsAnalyzer.NewsAnalyzerProperties;
import newsAnalyzer.dataModel.Facet;
import newsAnalyzer.dataModel.NormalizedTag;
import newsAnalyzer.dataModel.TagDraft;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

/**
 *
 * @author rcastari
 */
public class ReferenceFacetPredictor {

    protected static String DBpediaURL = "http://dbpedia.org/sparql?query=";
    protected static String _RESOURCE_NAME_ = "_RESOURCE_NAME_";
    protected static String baseDBpediaSPARQLquery =
              "PREFIX rdf:<http://www.w3.org/1999/02/22-rdf-syntax-ns#> " +
              "SELECT DISTINCT ?Concept " +
              "WHERE { <http://dbpedia.org/resource/_RESOURCE_NAME_> rdf:type ?Concept }";
    protected static String baseFormat = "&format=application/sparql-results+xml";

    protected static ReferenceFacetPredictor fp = null;

    public static ReferenceFacetPredictor getInstance(){
        if(fp == null){
            fp = new ReferenceFacetPredictor();
        }
        return fp;
    }

    protected ReferenceFacetPredictor(){
        String wnDbDir = NewsAnalyzerProperties.getInstance().getProperty(NewsAnalyzerProperties.WORDNET_DB_DIR);
        System.setProperty("wordnet.database.dir", wnDbDir);
    }

    protected Facet dbpediaSuggestion(String resourceName) throws Exception {

        /*
         * Encoding Scheme of DBpedia URIs: http://wiki.dbpedia.org/URIencoding?v=z6z
         *
            The encoding scheme of DBpedia URIs tries to be as close as possible to the Wikipedia title encoding scheme.

                The alphanumeric characters “a” through “z”, “A” through “Z” and “0” through “9” remain the same.
                The following special characters remain the same.
                    . (dot)
                    - (dash)
                    * (star)
                    / (slash)
                    : (colon)
                    _ (underscore)
                    , (comma)
                    & (ampersand)
                The space character ' ' is converted into an underscore character '_'.
                    Multiple underscores are collapsed into one.
                All other characters are unsafe and are first converted into one or more bytes using UTF-8 encoding. Then each byte is represented by the 3-character string "%xy", where xy is the two-digit hexadecimal representation of the byte.

            Please note that within the Internationalization efforts, the encoding scheme might change.
         *
         */

        String encodedResourceName = resourceName;
        encodedResourceName = encodedResourceName.replaceAll(" +", "_");
        encodedResourceName = encodedResourceName.replaceAll("_+", "_");
        encodedResourceName = URLEncoder.encode(encodedResourceName, "UTF-8");

        Facet facet = null;

        try {

            String query = baseDBpediaSPARQLquery.replace(_RESOURCE_NAME_, encodedResourceName);
            query = DBpediaURL + URLEncoder.encode(query, "UTF-8");
            query += baseFormat;

            URL url = new URL(query);
            HttpURLConnection httpConn = (HttpURLConnection)url.openConnection();

            if(NewsAnalyzerProperties.getInstance().debug()){
                System.out.println(httpConn);
            }

            InputStreamReader inputStreamReader = null;

            InputSource inputSource = null;

            boolean success = false;

            int n = 0;

            while (!success && n < 10) {
                try {
                    inputStreamReader = new InputStreamReader(httpConn.getInputStream());
                    inputSource = new InputSource(inputStreamReader);
                    success = true;
                } catch (IOException ex) {
                    String msg = "";
                    String aux;
                    BufferedReader msgr = new BufferedReader(new InputStreamReader(httpConn.getErrorStream()));
                    while((aux = msgr.readLine()) != null){
                        msg += aux;
                    }
                    Logger.getLogger(ReferenceFacetPredictor.class.getName()).log(Level.SEVERE, msg, ex);
                    n++;
                }
            }

            if (!success) {
                //TODO customize exception
                throw new Exception("Unable to access the DBpedia API or Malformed Query");
            }

            DOMParser parser = new DOMParser();

            parser.parse(inputSource);
            inputStreamReader.close();
            httpConn.disconnect();

            Document document = parser.getDocument();

            NodeList itemList = document.getElementsByTagName("result");

            boolean found = false;

            for(int i = 0; i < itemList.getLength() && !found ; i++){

                Node item = itemList.item(i);

                if (item.getNodeType() == Node.ELEMENT_NODE) {
                        Element resultItem = (Element) item;
                        NodeList resultUriList = resultItem.getElementsByTagName("uri");
                        if (resultUriList.getLength() > 0) {
                            Node resultUri = resultUriList.item(0);
                            NodeList resultUriChildNodes = resultUri.getChildNodes();
                            if (resultUriChildNodes.getLength() > 0) {
                                Node uriValue = resultUriChildNodes.item(0);
                                String uri = uriValue.getTextContent();
                                if(NewsAnalyzerProperties.getInstance().debug())
                                    System.out.println(uriValue.getTextContent());
                                String uriLower = uri.toLowerCase();
                                if(uriLower.endsWith("/place") || uriLower.endsWith("/location")){
                                    if(NewsAnalyzerProperties.getInstance().debug())
                                        System.out.println("\tPlace FOUND!!!");
                                    facet = Facet.GEOGRAPHIC_LOCATION;
                                    found = true;
                                }
                                if(uriLower.endsWith("/person") || uriLower.endsWith("/people")){
                                    if(NewsAnalyzerProperties.getInstance().debug())
                                        System.out.println("\tPerson FOUND!!!");
                                    facet = Facet.PERSON;
                                    found = true;
                                }
                                if(uriLower.endsWith("/organization") || uriLower.endsWith("/organisation")){
                                    if(NewsAnalyzerProperties.getInstance().debug())
                                        System.out.println("\tOrganization FOUND!!!");
                                    facet = Facet.ORGANIZATION;
                                    found = true;
                                }
                                if(uriLower.endsWith("/periodoftime")){
                                    if(NewsAnalyzerProperties.getInstance().debug())
                                        System.out.println("\tPeriod of Time FOUND!!!");
                                    facet = Facet.TIME_RELATED;
                                    found = true;
                                }

                            }
                        }
                }
            }

        } catch (MalformedURLException ex) {
            Logger.getLogger(ReferenceFacetPredictor.class.getName()).log(Level.SEVERE, null, ex);
        } catch (UnsupportedEncodingException ex) {
            Logger.getLogger(ReferenceFacetPredictor.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(ReferenceFacetPredictor.class.getName()).log(Level.SEVERE, null, ex);
        }

        return facet;

    }


    protected Facet wordNetSuggestion(String tagName){
        
        //WordNet hypernyms/hyponyms relationship (IS A relationship)

        Facet facet = null;

        NounSynset nounSynset = null;

        WordNetDatabase database = WordNetDatabase.getFileInstance();
        Synset[] synsets = database.getSynsets(tagName);

        if(synsets.length>0){
            int t = 0;
            while(nounSynset==null&&t<synsets.length){
                try{
                    nounSynset = (NounSynset)synsets[t];
                }catch(ClassCastException ex){
                    t++;
                }
            }

            boolean found = false;

            while(nounSynset!=null && !found){
                String[] wordForms = nounSynset.getWordForms();

                if (wordForms[0].equalsIgnoreCase("person")) {
                    facet = Facet.PERSON;
                    found = true;
                } else if (wordForms[0].equalsIgnoreCase("organization") || wordForms[0].equalsIgnoreCase("organisation")) {
                    facet = Facet.ORGANIZATION;
                    found = true;
                } else if (wordForms[0].equalsIgnoreCase("time period")) {
                    facet = Facet.TIME_RELATED;
                    found = true;
                } else if (wordForms[0].equalsIgnoreCase("location") || wordForms[0].equalsIgnoreCase("place")) {
                    facet = Facet.GEOGRAPHIC_LOCATION;
                    found = true;
                }

                if(NewsAnalyzerProperties.getInstance().debug())
                    System.out.println("\t\t" + wordForms[0]);

                nounSynset = this.getHypernym(nounSynset);
            }
        }

        return facet;
    }

    public Facet suggest(String tagName) throws Exception {
        Facet facet = this.dbpediaSuggestion(tagName);
        if(facet == null){
            facet= this.wordNetSuggestion(tagName);
            if(facet == null){
                facet = Facet.DESCRIPTIVE_SUBJECT;
            }
        }
        return facet;
    }

    private NounSynset getHypernym(NounSynset nounSynset){
        NounSynset result = null;
        
        NounSynset[] hypernyms = nounSynset.getInstanceHypernyms();
        
        if(hypernyms.length==0){
            hypernyms = nounSynset.getHypernyms();
        }

        if(hypernyms.length > 0){
            result = hypernyms[0];
        }
        return result;
    }

    public static void main (String args[]){

        /*String[] s = {
            "MCDONALD’S CORPORATION",
            "JAPAN",
            "REPUBLICAN PARTY",
            "YUCCA MOUNTAIN (NEV)",
            "NUCLEAR REGULATORY COMMISSION",
            "GEORGIA",
            "HENRY, THIERRY",
            "BOSTON RED SOX",
            "TORONTO BLUE JAYS",
            "NEW YORK YANKEES",
            "TURKEY",
            "MIDDLE AGE"
        };
         *
         */

        String[] s = {
                "URBAN AREAS",
                "ARCHITECTURE",
                "RACE AND ETHNICITY",
                "BLACKS",
                "CARBON DIOXIDE",
                "GREENHOUSE GAS EMISSIONS",
                "ENVIRONMENT",
                "CHILDREN AND YOUTH",
                "WEIGHT",
                "MCDONALD’S CORPORATION",
                "JAPAN",
                "HOUSE OF REPRESENTATIVES",
                "CONSUMER FINANCIAL PROTECTION BUREAU",
                "REPUBLICAN PARTY",
                "YUCCA MOUNTAIN (NEV)",
                "NUCLEAR REGULATORY COMMISSION",
                "ART",
                "MUSEUMS",
                "GEORGIA",
                "REVIEWS",
                "E3 MEDIA AND BUSINESS SUMMIT",
                "COMPUTER AND VIDEO GAMES",
                "MUSIC",
                "CLASSICAL MUSIC",
                "NEW YORK PHILHARMONIC",
                "TELEVISION",
                "LIFETIME TELEVISION",
                "DANCING",
                "ROYAL DANISH BALLET",
                "COLLEGES AND UNIVERSITIES",
                "SPAIN",
                "SONY CORPORATION",
                "SOCCER",
                "HENRY THIERRY",
                "NEW ENGLAND REVOLUTION",
                "NEW YORK RED BULLS",
                "HORSE RACING",
                "BELMONT STAKES",
                "FRANCE",
                "BASKETBALL",
                "FOOTBALL",
                "WOODS, TIGER",
                "JAMES, LEBRON",
                "OHIO STATE UNIVERSITY",
                "VANCOUVER CANUCKS",
                "DALLAS MAVERICKS",
                "MIAMI HEAT",
                "SKIING",
                "UTAH",
                "AUTOMOBILE RACING"
            };

        for (int i = 0; i < s.length; i++) {
            try {
                System.out.println("Tag "+(i+1)+": " + s[i]);
                String nt = ReferenceTagNormalizer.normalize(s[i]);
                Facet f = null;
                if (nt != null) {
                    f = getInstance().suggest(nt);
                } else {
                    nt = "";
                }
                TagDraft td = new TagDraft();
                td.setName(s[i]);
                NormalizedTag nt2 = TagNormalizer.normalize(td);
                String fName = (f==null)?"":f.name();
                String nt2Name = (nt2==null)?"":nt2.getName();
                String f2Name = (nt2==null)?"":((nt2.getFacet()==null)?"":nt2.getFacet().name());

                //String dbfName = (dbf==null)?"":dbf.name();
                //String wnfName = (wnf==null)?"":wnf.name();

                System.out.println("\tNormalized Tag: " + nt);
                System.out.println("\tSuggested facet: " + fName);
                System.out.println("\tNormalized Tag 2: " + nt2Name);
                System.out.println("\tSuggested facet 2: " + f2Name);
                //System.out.println("\t\tFacet suggested by DBpedia: " + dbfName);
                //System.out.println("\t\tFacet suggested by WordNet: " + wnfName);


            }
            /*
            try {
            System.out.println("Facets suggested by DBpedia:");
            System.out.println("\tFacet suggested for 'Socrates': "+getInstance().dbpediaSuggestion("Socrates"));
            System.out.println("\tFacet suggested for 'New York': "+getInstance().dbpediaSuggestion("New York"));
            System.out.println("\tFacet suggested for 'Middle age': "+getInstance().dbpediaSuggestion("Middle age"));
            System.out.println("\tFacet suggested for 'United Nations': "+getInstance().dbpediaSuggestion("United Nations"));
            System.out.println("\tFacet suggested for 'Apple': "+getInstance().dbpediaSuggestion("Apple"));
            System.out.println("Facets suggested by WordNet:");
            System.out.println("\tFacet suggested for 'Socrates': "+getInstance().wordNetSuggestion("Socrates"));
            System.out.println("\tFacet suggested for 'New York': "+getInstance().wordNetSuggestion("New York"));
            System.out.println("\tFacet suggested for 'Middle age': "+getInstance().wordNetSuggestion("Middle age"));
            System.out.println("\tFacet suggested for 'United Nations': "+getInstance().wordNetSuggestion("United Nations"));
            System.out.println("\tFacet suggested for 'Apple': "+getInstance().wordNetSuggestion("Apple"));
            System.out.println("Facets suggested:");
            System.out.println("\tFacet suggested for 'Socrates': "+getInstance().suggest("Socrates"));
            System.out.println("\tFacet suggested for 'New York': "+getInstance().suggest("New York"));
            System.out.println("\tFacet suggested for 'Middle age': "+getInstance().suggest("Middle age"));
            System.out.println("\tFacet suggested for 'United Nations': "+getInstance().suggest("United Nations"));
            System.out.println("\tFacet suggested for 'Apple': "+getInstance().suggest("Apple"));
            } catch (Exception ex) {
            Logger.getLogger(ReferenceFacetPredictor.class.getName()).log(Level.SEVERE, null, ex);
            }
             *
             *
             */ catch (Exception ex) {
                Logger.getLogger(ReferenceFacetPredictor.class.getName()).log(Level.SEVERE, null, ex);
            }
            }




        /*

        try {
            System.out.println("Facets suggested by DBpedia:");
            System.out.println("\tFacet suggested for 'Socrates': "+getInstance().dbpediaSuggestion("Socrates"));
            System.out.println("\tFacet suggested for 'New York': "+getInstance().dbpediaSuggestion("New York"));
            System.out.println("\tFacet suggested for 'Middle age': "+getInstance().dbpediaSuggestion("Middle age"));
            System.out.println("\tFacet suggested for 'United Nations': "+getInstance().dbpediaSuggestion("United Nations"));
            System.out.println("\tFacet suggested for 'Apple': "+getInstance().dbpediaSuggestion("Apple"));
            System.out.println("Facets suggested by WordNet:");
            System.out.println("\tFacet suggested for 'Socrates': "+getInstance().wordNetSuggestion("Socrates"));
            System.out.println("\tFacet suggested for 'New York': "+getInstance().wordNetSuggestion("New York"));
            System.out.println("\tFacet suggested for 'Middle age': "+getInstance().wordNetSuggestion("Middle age"));
            System.out.println("\tFacet suggested for 'United Nations': "+getInstance().wordNetSuggestion("United Nations"));
            System.out.println("\tFacet suggested for 'Apple': "+getInstance().wordNetSuggestion("Apple"));
            System.out.println("Facets suggested:");
            System.out.println("\tFacet suggested for 'Socrates': "+getInstance().suggest("Socrates"));
            System.out.println("\tFacet suggested for 'New York': "+getInstance().suggest("New York"));
            System.out.println("\tFacet suggested for 'Middle age': "+getInstance().suggest("Middle age"));
            System.out.println("\tFacet suggested for 'United Nations': "+getInstance().suggest("United Nations"));
            System.out.println("\tFacet suggested for 'Apple': "+getInstance().suggest("Apple"));

        } catch (Exception ex) {
            Logger.getLogger(ReferenceFacetPredictor.class.getName()).log(Level.SEVERE, null, ex);
        }
         *
         *
         */
    }

}
