/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package newsAnalyzer.analyzer.utils;

import com.sun.org.apache.xerces.internal.parsers.DOMParser;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.util.logging.Level;
import java.util.logging.Logger;
import newsAnalyzer.NewsAnalyzerProperties;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

/**
 *
 * @author rcastari
 */
public class ReferenceTagNormalizer {

    private static final String baseURL = "http://en.wikipedia.org/w/api.php?action=opensearch&search=";
    private static final String baseFormat = "&format=xml";

    private static class WikipediaEntry{
        private String name;
        private String url;

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }

        public String getUrl() {
            return url;
        }

        public void setUrl(String url) {
            this.url = url;
        }

    }

    public static String normalize(String tagName) throws Exception {
        String nt = null;

        WikipediaEntry we = search(tagName);

        if(we!=null){
            nt=we.name;
        }

        return nt;
    }

    private static WikipediaEntry search(String tagName) throws Exception {

        WikipediaEntry wikipediaEntry = null;

        String query;
        try {
            query = baseURL + URLEncoder.encode(tagName, "UTF-8") + baseFormat;
        } catch (UnsupportedEncodingException ex) {
            Logger.getLogger(ReferenceTagNormalizer.class.getName()).log(Level.SEVERE, null, ex);
            throw ex;
        }

        try {

            URL url = new URL(query);
            HttpURLConnection httpConn = (HttpURLConnection)url.openConnection();

            if(NewsAnalyzerProperties.getInstance().debug()){
                System.out.println(httpConn);
            }

            boolean success = false;

            int n = 0;

            InputStreamReader inputStreamReader = null;

            InputSource inputSource = null;

            while (!success && n < 10) {
                try {
                    inputStreamReader = new InputStreamReader(httpConn.getInputStream());
                    inputSource = new InputSource(inputStreamReader);
                    success = true;
                } catch (IOException ex) {
                    //TODO tractar excepció, aturar l'actualització de la bbdd?
                    Logger.getLogger(ReferenceTagNormalizer.class.getName()).log(Level.SEVERE, null, ex);
                    if (n < 5) {
                        n++;
                    } else {
                        throw ex;
                    }
                }
            }

            if (!success) {
                throw new Exception("Unable to connect to Wikipeida API or Bad Query Format, query: " + query);
            }

            DOMParser parser = new DOMParser();

            parser.parse(inputSource);
            inputStreamReader.close();
            httpConn.disconnect();

            Document document = parser.getDocument();

            NodeList itemList = document.getElementsByTagName("Item");

            int numberOfItems = itemList.getLength();

            if (numberOfItems > 0) {
                if (numberOfItems == 1) {
                    Node item = itemList.item(0);
                    if (item.getNodeType() == Node.ELEMENT_NODE) {
                        Element wikipediaItem = (Element) item;
                        NodeList urlList = wikipediaItem.getElementsByTagName("Url");
                        NodeList textList = wikipediaItem.getElementsByTagName("Text");
                        if (urlList.getLength() > 0 && textList.getLength() > 0) {
                            Node itemUrl = urlList.item(0);
                            Node itemText = textList.item(0);
                            NodeList itemUrlChildNodes = itemUrl.getChildNodes();
                            NodeList itemTextChildNodes = itemText.getChildNodes();
                            if (itemUrlChildNodes.getLength() > 0 && itemTextChildNodes.getLength() > 0) {
                                Node urlValue = itemUrlChildNodes.item(0);
                                Node textValue = itemTextChildNodes.item(0);
                                wikipediaEntry = new WikipediaEntry();
                                wikipediaEntry.setName(textValue.getNodeValue());
                                wikipediaEntry.setUrl(urlValue.getNodeValue());
                            }
                        }
                    }
                } else {
                    //TODO disambiguate, now only takes the first item
                    Node item = itemList.item(0);
                    if (item.getNodeType() == Node.ELEMENT_NODE) {
                        Element wikipediaItem = (Element) item;
                        NodeList urlList = wikipediaItem.getElementsByTagName("Url");
                        NodeList textList = wikipediaItem.getElementsByTagName("Text");
                        if (urlList.getLength() > 0 && textList.getLength() > 0) {
                            Node itemUrl = urlList.item(0);
                            Node itemText = textList.item(0);
                            NodeList itemUrlChildNodes = itemUrl.getChildNodes();
                            NodeList itemTextChildNodes = itemText.getChildNodes();
                            if (itemUrlChildNodes.getLength() > 0 && itemTextChildNodes.getLength() > 0) {
                                Node urlValue = itemUrlChildNodes.item(0);
                                Node textValue = itemTextChildNodes.item(0);
                                wikipediaEntry = new WikipediaEntry();
                                wikipediaEntry.setName(textValue.getNodeValue());
                                wikipediaEntry.setUrl(urlValue.getNodeValue());
                            }
                        }
                    }
                }
            } else {
                //TODO what to do if the item isn't found at wikipedia
                //DO nothing
            }


        } catch (MalformedURLException ex) {
            Logger.getLogger(ReferenceTagNormalizer.class.getName()).log(Level.SEVERE, null, ex);
        }

        return wikipediaEntry;
    }

    public static void main(String args[]) {
        try {
            String[] s = {
                "URBAN AREAS",
                "ARCHITECTURE",
                "RACE AND ETHNICITY",
                "BLACKS",
                "CARBON DIOXIDE",
                "GREENHOUSE GAS EMISSIONS",
                "ENVIRONMENT",
                "CHILDREN AND YOUTH",
                "WEIGHT",
                "MCDONALD’S CORPORATION",
                "JAPAN",
                "HOUSE OF REPRESENTATIVES",
                "CONSUMER FINANCIAL PROTECTION BUREAU",
                "REPUBLICAN PARTY",
                "YUCCA MOUNTAIN (NEV)",
                "NUCLEAR REGULATORY COMMISSION",
                "ART",
                "MUSEUMS",
                "GEORGIA",
                "REVIEWS",
                "E3 MEDIA AND BUSINESS SUBMMIT",
                "COUMPUTERS AND VIDEO GAMES",
                "MUSIC",
                "CLASSICAL MUSIC",
                "NEW YORK PHILHARMONIC",
                "TELEVISION",
                "LIFETIME TELEVISION",
                "DANCING",
                "ROYAL DANISH BALLET",
                "COLLEGES AND UNIVERSITIES",
                "SPAIN",
                "SONY CORPORATION",
                "SOCCER",
                "HENRY THIERRY",
                "NEW ENGLAND REVOLUTION",
                "NEW YORK RED BULLS",
                "HORSE RACING",
                "BELMONT STAKES",
                "FRANCE",
                "BASKETBALL",
                "FOOTBALL",
                "WOODS, TIGER",
                "JAMES, LEBRON",
                "OHIO STATE UNIVERSITY",
                "VANCOUVER CANUCKS",
                "DALLAS MAVERICKS",
                "MIAMI HEAT",
                "SKIING",
                "UTAH",
                "AUTOMOBILE RACING"
            };
            for(int i=0; i<s.length; i++){
                System.out.println("Tag: "+s[i]);
                WikipediaEntry we = search(s[i]);
                String nt = "";
                if(we!=null){
                    nt=we.name;
                }
                System.out.println("\tNormalized Tag: "+nt);
            }

        } catch (Exception ex) {
            Logger.getLogger(ReferenceTagNormalizer.class.getName()).log(Level.SEVERE, null, ex);
        }

    }
 
}
