/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package newsAnalyzer.analyzer.utils.frontEnds.dbpedia;

import com.sun.org.apache.xerces.internal.parsers.DOMParser;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.util.logging.Level;
import java.util.logging.Logger;
import newsAnalyzer.NewsAnalyzerProperties;
import newsAnalyzer.dataModel.Facet;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

/**
 *
 * @author rcastari
 */
public class DBpediaFrontEnd {

    protected static final int MAX_COUNT = 10;
    protected static String DBpediaURL = "http://dbpedia.org/sparql?query=";
    protected static String _RESOURCE_NAME_ = "_RESOURCE_NAME_";
    protected static String baseDBpediaSPARQLquery =
            "PREFIX rdf:<http://www.w3.org/1999/02/22-rdf-syntax-ns#> "
            + "PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>"
            + "SELECT DISTINCT ?Concept ?Label "
            + "WHERE {"
            + " <http://dbpedia.org/resource/_RESOURCE_NAME_> rdf:type ?Concept "
            + " . ?Concept rdfs:label ?Label "
            + "FILTER ( LANG ( ?Label) = \"en\" || LANG ( ?Label) = \"\")"
            + "}";
    protected static String baseFormat = "&format=application/sparql-results+xml";
    protected static DBpediaFrontEnd dbpfe = null;

    public static DBpediaFrontEnd getInstance() {
        if (dbpfe == null) {
            dbpfe = new DBpediaFrontEnd();
        }
        return dbpfe;
    }

    protected DBpediaFrontEnd() {
        String wnDbDir = NewsAnalyzerProperties.getInstance().getProperty(NewsAnalyzerProperties.WORDNET_DB_DIR);
        System.setProperty("wordnet.database.dir", wnDbDir);
    }

    public Facet suggestFacet(String resourceName) throws Exception {

        /*
         * Encoding Scheme of DBpedia URIs: http://wiki.dbpedia.org/URIencoding?v=z6z
         *
        The encoding scheme of DBpedia URIs tries to be as close as possible to the Wikipedia title encoding scheme.

        The alphanumeric characters “a” through “z”, “A” through “Z” and “0” through “9” remain the same.
        The following special characters remain the same.
        . (dot)
        - (dash)
         * (star)
        / (slash)
        : (colon)
        _ (underscore)
        , (comma)
        & (ampersand)
        The space character ' ' is converted into an underscore character '_'.
        Multiple underscores are collapsed into one.
        All other characters are unsafe and are first converted into one or more bytes using UTF-8 encoding. Then each byte is represented by the 3-character string "%xy", where xy is the two-digit hexadecimal representation of the byte.

        Please note that within the Internationalization efforts, the encoding scheme might change.
         *
         */

        String encodedResourceName = resourceName;
        encodedResourceName = encodedResourceName.replaceAll(" +", "_");
        encodedResourceName = encodedResourceName.replaceAll("_+", "_");
        encodedResourceName = URLEncoder.encode(encodedResourceName, "UTF-8");

        Facet facet = null;

        try {

            String query = baseDBpediaSPARQLquery.replace(_RESOURCE_NAME_, encodedResourceName);
            query = DBpediaURL + URLEncoder.encode(query, "UTF-8");
            query += baseFormat;

            URL url = new URL(query);
            HttpURLConnection httpConn = (HttpURLConnection) url.openConnection();

            if (NewsAnalyzerProperties.getInstance().debug()) {
                System.out.println(httpConn);
            }

            InputStreamReader inputStreamReader = null;

            InputSource inputSource = null;

            boolean success = false;

            int n = 0;

            while (!success && n < 10) {
                try {
                    inputStreamReader = new InputStreamReader(httpConn.getInputStream());
                    inputSource = new InputSource(inputStreamReader);
                    success = true;
                } catch (IOException ex) {
                    String msg = "";
                    String aux;
                    BufferedReader msgr = new BufferedReader(new InputStreamReader(httpConn.getErrorStream()));
                    while ((aux = msgr.readLine()) != null) {
                        msg += aux;
                    }
                    Logger.getLogger(DBpediaFrontEnd.class.getName()).log(Level.SEVERE, msg, ex);
                    n++;
                }
            }

            if (!success) {
                //TODO customize exception
                throw new Exception("Unable to access the DBpedia API or Malformed Query");
            }

            DOMParser parser = new DOMParser();

            parser.parse(inputSource);
            inputStreamReader.close();
            httpConn.disconnect();

            Document document = parser.getDocument();

            NodeList itemList = document.getElementsByTagName("result");

            boolean found = false;

            for (int i = 0; i < itemList.getLength() && !found; i++) {

                Node item = itemList.item(i);

                if (item.getNodeType() == Node.ELEMENT_NODE) {
                    Element resultItem = (Element) item;
                    NodeList resultLiteralList = resultItem.getElementsByTagName("literal");
                    if (resultLiteralList.getLength() > 0) {
                        Node resultLiteral = resultLiteralList.item(0);
                        NodeList resultUriChildNodes = resultLiteral.getChildNodes();
                        if (resultUriChildNodes.getLength() > 0) {
                            Node uriValue = resultUriChildNodes.item(0);
                            String uri = uriValue.getTextContent();
                            if (NewsAnalyzerProperties.getInstance().debug()) {
                                System.out.println(uriValue.getTextContent());
                            }
                            String uriLower = uri.toLowerCase();
                            if (uriLower.equalsIgnoreCase("place") || uriLower.equalsIgnoreCase("location")) {
                                if (NewsAnalyzerProperties.getInstance().debug()) {
                                    System.out.println("\tPlace FOUND!!!");
                                }
                                facet = Facet.GEOGRAPHIC_LOCATION;
                                found = true;
                            }
                            if (uriLower.equalsIgnoreCase("person") || uriLower.equalsIgnoreCase("people")) {
                                if (NewsAnalyzerProperties.getInstance().debug()) {
                                    System.out.println("\tPerson FOUND!!!");
                                }
                                facet = Facet.PERSON;
                                found = true;
                            }
                            if (uriLower.equalsIgnoreCase("organization") || uriLower.equalsIgnoreCase("organisation")) {
                                if (NewsAnalyzerProperties.getInstance().debug()) {
                                    System.out.println("\tOrganization FOUND!!!");
                                }
                                facet = Facet.ORGANIZATION;
                                found = true;
                            }
                            if (uriLower.equalsIgnoreCase("periodoftime")) {
                                if (NewsAnalyzerProperties.getInstance().debug()) {
                                    System.out.println("\tPeriod of Time FOUND!!!");
                                }
                                facet = Facet.TIME_RELATED;
                                found = true;
                            }

                        }
                    }
                }
            }

        } catch (MalformedURLException ex) {
            Logger.getLogger(DBpediaFrontEnd.class.getName()).log(Level.SEVERE, null, ex);
        } catch (UnsupportedEncodingException ex) {
            Logger.getLogger(DBpediaFrontEnd.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(DBpediaFrontEnd.class.getName()).log(Level.SEVERE, null, ex);
        }

        return facet;

    }
    /*
    public Facet suggestFacet(String tagName, String tagCathegory) throws Exception{
    Facet facet = null;

    String[] hypernyms = WordNetFrontEnd.getInstance().getHypernyms(tagCathegory);

    if(hypernyms.length==0){
    hypernyms = getHypernyms(tagCathegory);
    }

    int cathLevel=0;
    int nounLevel=0;
    boolean end = false;
    String[] currentHypernyms = hypernyms;

    for(int n=0; n < hypernyms.length && !end; n++){
    System.out.println("Cathegory hypernym: "+hypernyms[n]);
    for(int level)

    }

    return facet;
    }
     */

    /*
    private String[] getHypernyms(String noun) throws Exception {

        List<String> hypernyms = new ArrayList<String>();

        String encodedNoun = noun;
        encodedNoun = encodedNoun.replaceAll(" +", "_");
        encodedNoun = encodedNoun.replaceAll("_+", "_");
        //encodedNoun = URLEncoder.encode(encodedNoun, "UTF-8");

        try {

            String query = baseDBpediaSPARQLquery.replace(_RESOURCE_NAME_, encodedNoun);
            query = DBpediaURL + URLEncoder.encode(query, "UTF-8");
            query += baseFormat;

            URL url = new URL(query);

            HttpURLConnection httpConn = null;

            InputStreamReader inputStreamReader = null;

            InputSource inputSource = null;

            boolean success = false;

            int n = 0;

            while (!success && n < 10) {
                try {

                    httpConn = (HttpURLConnection) url.openConnection();

                    if (NewsAnalyzerProperties.getInstance().debug()) {
                        System.out.println(httpConn);
                    }

                    inputStreamReader = new InputStreamReader(httpConn.getInputStream());
                    inputSource = new InputSource(inputStreamReader);
                    success = true;
                } catch (IOException ex) {
                    String msg = "";
                    String aux;
                    BufferedReader msgr = new BufferedReader(new InputStreamReader(httpConn.getErrorStream()));
                    while ((aux = msgr.readLine()) != null) {
                        msg += aux;
                    }
                    httpConn.disconnect();
                    Logger.getLogger(DBpediaFrontEnd.class.getName()).log(Level.SEVERE, msg, ex);
                    n++;
                }
            }

            if (!success) {
                //TODO customize exception
                throw new Exception("Unable to access the DBpedia API or Malformed Query");
            }

            DOMParser parser = new DOMParser();

            parser.parse(inputSource);
            inputStreamReader.close();
            httpConn.disconnect();

            Document document = parser.getDocument();

            NodeList itemList = document.getElementsByTagName("result");

            boolean found = false;

            for (int i = 0; i < itemList.getLength() && !found; i++) {

                Node item = itemList.item(i);

                if (item.getNodeType() == Node.ELEMENT_NODE) {
                    Element resultItem = (Element) item;
                    NodeList resultLiteralList = resultItem.getElementsByTagName("literal");
                    if (resultLiteralList.getLength() > 0) {
                        Node resultLiteral = resultLiteralList.item(0);
                        NodeList resultLiteralChildNodes = resultLiteral.getChildNodes();
                        if (resultLiteralChildNodes.getLength() > 0) {
                            Node literalValue = resultLiteralChildNodes.item(0);
                            String literal = literalValue.getTextContent();
                            if (NewsAnalyzerProperties.getInstance().debug()) {
                                System.out.println(literalValue.getTextContent());
                            }
                            hypernyms.add(literal);
                        }
                    }
                }
            }

        } catch (MalformedURLException ex) {
            Logger.getLogger(DBpediaFrontEnd.class.getName()).log(Level.SEVERE, null, ex);
        } catch (UnsupportedEncodingException ex) {
            Logger.getLogger(DBpediaFrontEnd.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(DBpediaFrontEnd.class.getName()).log(Level.SEVERE, null, ex);
        }

        String[] result = hypernyms.toArray(new String[0]);

        return result;
    }
     *
     */

    /*
    public InheritedHypernymsTree getInheritedHypernymsListImproved2(String noun) throws Exception {

        InheritedHypernymsTree hypernyms = new InheritedHypernymsTree();

        Queue<String> elements = new LinkedList<String>();

        hypernyms.add(noun, null);
        elements.add(noun);

        int count = 0;
        
        while (!elements.isEmpty() && count < MAX_COUNT) {

            //TODO eliminar això if(elements.peek()==null){System.out.println("NULL HERE!");}

            String parent = elements.remove();
            
            String hypernymsArray[] = WordNetFrontEnd.getInstance().getHypernymsArray(parent);

            if (hypernymsArray.length == 0) {
                hypernymsArray = this.getHypernyms(parent);
                for (int j = 0; j < hypernymsArray.length; j++) {

                    String hypernym = hypernymsArray[j];
                    WikipediaEntry we = WikipediaFrontEnd.search(hypernym);

                    if (we != null) {
                        hypernym = we.getName();
                    }

                    if (hypernyms.add(hypernym, parent) == 1) {
                        elements.add(hypernym);
                    }
                }

            } else {
                if(hypernyms.add(hypernymsArray[0], parent)==1){
                    elements.add(hypernymsArray[0]);
                }
            }

            count++;

        }

        if (!elements.isEmpty()) {
            elements.clear();
            elements = null;
        }

        return hypernyms;
    }

    */

    public static void main(String args[]) {
        try {
            System.out.println("Facets suggested by DBpedia:");
            System.out.println("Suggest Facet for 'Socrates': ");
            System.out.println("Facet suggested for 'Socrates': " + getInstance().suggestFacet("Socrates"));
            System.out.println("Suggest Facet for 'New York': ");
            System.out.println("Facet suggested for 'New York': " + getInstance().suggestFacet("New York"));
            System.out.println("Suggest Facet for 'Middle age': ");
            System.out.println("Facet suggested for 'Middle age': " + getInstance().suggestFacet("Middle age"));
            System.out.println("Suggest Facet for 'United Nations': ");
            System.out.println("Facet suggested for 'United Nations': " + getInstance().suggestFacet("United Nations"));
            System.out.println("Suggest Facet for 'Apple': ");
            System.out.println("Facet suggested for 'Apple': " + getInstance().suggestFacet("Apple"));
            /*
            System.out.println("\tFacet suggested for 'Socrates': "+getInstance().suggestFacet("Socrates","Phylosophy"));
            System.out.println("\tFacet suggested for 'New York': "+getInstance().suggestFacet("New York","US"));
            System.out.println("\tFacet suggested for 'Middle age': "+getInstance().suggestFacet("Middle age","Era"));
            System.out.println("\tFacet suggested for 'United Nations': "+getInstance().suggestFacet("United Nations","World"));
            System.out.println("\tFacet suggested for 'Apple': "+getInstance().suggestFacet("Apple","Computers"));
             */

        } catch (Exception ex) {
            Logger.getLogger(DBpediaFrontEnd.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
}
