/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package newsAnalyzer.analyzer.utils.frontEnds.wikipedia;

import com.sun.org.apache.xerces.internal.parsers.DOMParser;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import newsAnalyzer.NewsAnalyzerProperties;
import newsAnalyzer.analyzer.utils.StringComparator;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

/**
 *
 * @author rcastari
 */
public class WikipediaFrontEnd {

    private static final String baseURL = "http://en.wikipedia.org/w/api.php?action=opensearch&search=";
    private static final String baseFormat = "&format=xml";

    public static List<WikipediaEntry> searchCandidates(String tagName) throws Exception {
        List<WikipediaEntry> list = new ArrayList<WikipediaEntry>();

        String query;

        try {
            query = baseURL + URLEncoder.encode(tagName, "UTF-8") + baseFormat;
        } catch (UnsupportedEncodingException ex) {
            Logger.getLogger(WikipediaFrontEnd.class.getName()).log(Level.SEVERE, null, ex);
            throw ex;
        }

        try {

            URL url = new URL(query);
            HttpURLConnection httpConn = (HttpURLConnection)url.openConnection();

            if(NewsAnalyzerProperties.getInstance().debug()){
                System.out.println(httpConn);
            }

            boolean success = false;

            int n = 0;

            InputStreamReader inputStreamReader = null;

            InputSource inputSource = null;

            while (!success && n < 10) {
                try {
                    inputStreamReader = new InputStreamReader(httpConn.getInputStream());
                    inputSource = new InputSource(inputStreamReader);
                    success = true;
                } catch (IOException ex) {
                    //TODO tractar excepció, aturar l'actualització de la bbdd?
                    Logger.getLogger(WikipediaFrontEnd.class.getName()).log(Level.SEVERE, null, ex);
                    if (n < 5) {
                        n++;
                    } else {
                        throw ex;
                    }
                }
            }

            if (!success) {
                throw new Exception("Unable to connect to Wikipeida API or Bad Query Format, query: " + query);
            }

            DOMParser parser = new DOMParser();

            parser.parse(inputSource);
            inputStreamReader.close();
            httpConn.disconnect();

            Document document = parser.getDocument();

            NodeList itemList = document.getElementsByTagName("Item");

            WikipediaEntry candidate = null;

            for (int i = 0; i < itemList.getLength(); i++) {

                Node item = itemList.item(i);

                candidate = parseWikipediaEntry(item);

                list.add(candidate);

            }

        } catch (MalformedURLException ex) {
            Logger.getLogger(WikipediaFrontEnd.class.getName()).log(Level.SEVERE, null, ex);
        }

        return list;
    }

    public static WikipediaEntry search(String tagName) throws Exception {

        List<WikipediaEntry> candidates = searchCandidates(tagName);

        WikipediaEntry bestCandidate = null;
        int bestScore = -1;

        boolean found = false;
            WikipediaEntry candidate = null;

        for (int i = 0; i < candidates.size() && !found; i++) {

            candidate = candidates.get(i);
            int score = StringComparator.score(candidate.getName(), tagName);

            if(bestCandidate == null){
                bestCandidate = candidate;
                bestScore = score;
            }else if (score > bestScore) {
                bestCandidate = candidate;
                if(score == 2){
                    found = true;
                }
            }

        }

        return bestCandidate;
    }

    /*

    public static WikipediaEntry search(String tagName, String cathegory) throws Exception {

        List<WikipediaEntry> candidates = searchCandidates(tagName);

        Integer score[] = new Integer[candidates.size()];

        WikipediaEntry bestCandidate = null;

        WikipediaEntry candidate = null;

        int min = 0;

        for (int i = 0; i < candidates.size(); i++) {

            candidate = candidates.get(i);

            score[i]=StringComparator.semanticDistanceBetweenNounAndCathegory(candidate.getName(), cathegory);

            if(score[i] != null){
                if(score[min]==null){
                    min = i;
                }else{
                    if(score[i]<score[min]){
                        min=i;
                    }
                }
            }
        }

        if(!candidates.isEmpty()){
            bestCandidate = candidates.get(min);
        }

        return bestCandidate;
    }
     *
     */
    
    private static WikipediaEntry parseWikipediaEntry(Node item) {
        WikipediaEntry we = null;
        if (item.getNodeType() == Node.ELEMENT_NODE) {
            Element wikipediaItem = (Element) item;
            NodeList urlList = wikipediaItem.getElementsByTagName("Url");
            NodeList textList = wikipediaItem.getElementsByTagName("Text");
            if (urlList.getLength() > 0 && textList.getLength() > 0) {
                Node itemUrl = urlList.item(0);
                Node itemText = textList.item(0);
                NodeList itemUrlChildNodes = itemUrl.getChildNodes();
                NodeList itemTextChildNodes = itemText.getChildNodes();
                if (itemUrlChildNodes.getLength() > 0 && itemTextChildNodes.getLength() > 0) {
                    Node urlValue = itemUrlChildNodes.item(0);
                    Node textValue = itemTextChildNodes.item(0);
                    we = new WikipediaEntry();
                    we.setName(textValue.getNodeValue());
                    we.setUrl(urlValue.getNodeValue());
                }
            }
        }
        return we;
    }

    public static void main(String args[]) {
        try {
            String word="Bacon";
            System.out.println("Searching word '"+word+"' on Wikipedia.");
            WikipediaEntry we = search(word);
            System.out.println("Result:");
            if(we!=null){
                System.out.println("\tName: "+we.getName());
                System.out.println("\tUrl: "+we.getUrl());
            }else{
                System.out.println("\tnull");
            }
/*
            String tagName="Leeds";
            String cathegory="sport";

            System.out.println("Searching word '"+tagName+"' with cathegory '"+cathegory+"' on Wikipedia.");
            WikipediaEntry bestCandidate = search(tagName,cathegory);
            System.out.println("Result:");
            if(bestCandidate!=null){
                System.out.println("\tName: "+bestCandidate.getName());
                System.out.println("\tUrl: "+bestCandidate.getUrl());
            }else{
                System.out.println("\tnull");
            }
            
  */

        } catch (Exception ex) {
            Logger.getLogger(WikipediaFrontEnd.class.getName()).log(Level.SEVERE, null, ex);
        }

    }
 
}
