/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package newsAnalyzer.analyzer.utils.frontEnds.wordNet;

import edu.smu.tspell.wordnet.NounSynset;
import edu.smu.tspell.wordnet.Synset;
import edu.smu.tspell.wordnet.WordNetDatabase;
import java.util.logging.Level;
import java.util.logging.Logger;
import newsAnalyzer.NewsAnalyzerProperties;
import newsAnalyzer.analyzer.utils.StringComparator;
import newsAnalyzer.dataModel.Facet;

/**
 *
 * @author rcastari
 */
public class WordNetFrontEnd {

    protected static WordNetFrontEnd wnfe = null;

    public static WordNetFrontEnd getInstance() {
        if (wnfe == null) {
            wnfe = new WordNetFrontEnd();
        }
        return wnfe;
    }

    protected WordNetFrontEnd() {
        String wnDbDir = NewsAnalyzerProperties.getInstance().getProperty(NewsAnalyzerProperties.WORDNET_DB_DIR);
        System.setProperty("wordnet.database.dir", wnDbDir);
    }

    protected NounSynset searchNounSynset(String noun) {
        NounSynset candidate = null;
        NounSynset bestCandidate = null;
        int bestScore = -1;

        WordNetDatabase database = WordNetDatabase.getFileInstance();
        Synset[] synsets = database.getSynsets(noun);

        boolean found = false;
        
        for (int i = 0; i < synsets.length && !found; i++) {
            if (synsets[i] instanceof NounSynset) {
                candidate = (NounSynset) synsets[i];
                int score = StringComparator.score(candidate.getWordForms()[0], noun);
                if(bestCandidate == null){
                    bestCandidate = candidate;
                    bestScore = score;
                }else if (score > bestScore) {
                    bestCandidate = candidate;
                    bestScore = score;
                    if(score == 2){
                        found = true;
                    }
                }
            }
        }
        return bestCandidate;
    }

    /*
    private NounSynset[] searchNounSynsetArray(String noun){
        NounSynset[] result = new NounSynset[0];
        NounSynset candidate;
        List<NounSynset> list = new ArrayList<NounSynset>();
        WordNetDatabase database = WordNetDatabase.getFileInstance();
        Synset[] synsets = database.getSynsets(noun);

        boolean found = false;

        for (int i = 0; i < synsets.length && !found; i++) {
            if (synsets[i] instanceof NounSynset) {
                candidate = (NounSynset) synsets[i];
                list.add(candidate);
            }
        }

        if(list.size()>0){
            result = new NounSynset[list.size()];
            list.toArray(result);
        }
        return result;
    }
     *
     */

    public String searchNoun(String noun) {
        String result = null;

        NounSynset bestCandidate = searchNounSynset(noun);

        if (bestCandidate != null) {
            result = bestCandidate.getWordForms()[0];
        }

        return result;
    }

    public Facet suggestFacet(String tagName) {

        //WordNet hypernyms/hyponyms relationship (IS A relationship)

        if (NewsAnalyzerProperties.getInstance().debug()) {
            System.out.println("\tSearching in WordNet for word '" + tagName + "':");
        }

        Facet facet = null;

        NounSynset nounSynset = searchNounSynset(tagName);

        boolean found = false;

        while (nounSynset != null && !found) {
            String[] wordForms = nounSynset.getWordForms();

            if (wordForms[0].equalsIgnoreCase("person")) {
                facet = Facet.PERSON;
                found = true;
            } else if (wordForms[0].equalsIgnoreCase("organization") || wordForms[0].equalsIgnoreCase("organisation")) {
                facet = Facet.ORGANIZATION;
                found = true;
            } else if (wordForms[0].equalsIgnoreCase("time period")) {
                facet = Facet.TIME_RELATED;
                found = true;
            } else if (wordForms[0].equalsIgnoreCase("location") || wordForms[0].equalsIgnoreCase("place")) {
                facet = Facet.GEOGRAPHIC_LOCATION;
                found = true;
            }

            if (NewsAnalyzerProperties.getInstance().debug()) {
                System.out.println("\t\t" + wordForms[0]);
            }

            nounSynset = this.getHypernym(nounSynset);
        }


        return facet;
    }

    private NounSynset getHypernym(NounSynset nounSynset) {
        NounSynset result = null;

        NounSynset[] hypernyms = nounSynset.getInstanceHypernyms();

        if (hypernyms.length == 0) {
            hypernyms = nounSynset.getHypernyms();
        }

        if (hypernyms.length > 0) {
            result = hypernyms[0];
        }
        return result;
    }


    /*
    private NounSynset[] getHypernyms(NounSynset nounSynset){
        
        NounSynset[] hypernyms = new NounSynset[0];

        if (nounSynset != null) {
            hypernyms = nounSynset.getInstanceHypernyms();
            if (hypernyms.length == 0) {
                hypernyms = nounSynset.getHypernyms();
            }
        }

        return hypernyms;
    }
     *
     */

    /*
    public String[] getHypernymsArray(String noun){

        //Common

        String result[] = new String[0];
        
/*
         *
         *
        List<String> list = new ArrayList<String>();

        NounSynset synset[] = this.searchNounSynsetArray(noun);

        for(int j=0; j<synset.length; j++){
            
            NounSynset nounSynset = synset[j];
        
            NounSynset[] hypernyms = new NounSynset[0];

            if (nounSynset != null) {
                hypernyms = nounSynset.getInstanceHypernyms();
                if (hypernyms.length == 0) {
                    hypernyms = nounSynset.getHypernyms();
                }
            }

            if (hypernyms.length > 0) {
                for (int i = 0; i < hypernyms.length; i++) {
                    list.add(hypernyms[i].getWordForms()[0]);
                }
            }

        }

        result = new String[list.size()];
        list.toArray(result);
         *
         *
*/
/*
        NounSynset nounSynset = this.searchNounSynset(noun);

        NounSynset[] hypernyms = this.getHypernyms(nounSynset);

        if (hypernyms.length > 0) {
            result = new String[hypernyms.length];
            for (int i = 0; i < hypernyms.length; i++) {
                result[i]=hypernyms[i].getWordForms()[0];
            }
        }

        //Common

        return result;
    }
    *
    */

    /*
    public InheritedHypernymsTree getInheritedHypernymsListImproved2(String noun) {

        InheritedHypernymsTree inheritedHypernyms = new InheritedHypernymsTree();

        Queue<NounSynset> hypernymsSynset = new LinkedList<NounSynset>();

        NounSynset nounSynset = searchNounSynset(noun);

        if (nounSynset != null) {
            try {
                inheritedHypernyms.add(nounSynset.getWordForms()[0], null);
                hypernymsSynset.add(nounSynset);
                while (!hypernymsSynset.isEmpty()) {
                    NounSynset parent = hypernymsSynset.remove();
                    NounSynset[] hypernymsArray = getHypernyms(parent);
                    if (hypernymsArray.length > 0) {
                        if(inheritedHypernyms.add(hypernymsArray[0].getWordForms()[0], parent.getWordForms()[0])==1){
                            hypernymsSynset.add(hypernymsArray[0]);
                        }
                    }
                }
            } catch (Exception ex) {
                Logger.getLogger(WordNetFrontEnd.class.getName()).log(Level.SEVERE, null, ex);
            }
        }

        return inheritedHypernyms;
    }
     * 
     */

    public static void main(String args[]) {
        try {
            System.out.println("Search word in WordNet:");
            System.out.println("\tSearch word 'World': " + getInstance().searchNoun("World"));
            System.out.println("\tSearch word 'NY': " + getInstance().searchNoun("NY"));
            System.out.println("\tSearch word 'Obama': " + getInstance().searchNoun("Obama"));
            System.out.println("Facets suggested by WordNet:");
            System.out.println("\tFacet suggested for 'Socrates': " + getInstance().suggestFacet("Socrates"));
            System.out.println("\tFacet suggested for 'World': " + getInstance().suggestFacet("World"));
            System.out.println("\tFacet suggested for 'Middle age': " + getInstance().suggestFacet("Middle age"));
            System.out.println("\tFacet suggested for 'United Nations': " + getInstance().suggestFacet("United Nations"));
            System.out.println("\tFacet suggested for 'Apple': " + getInstance().suggestFacet("Apple"));

        } catch (Exception ex) {
            Logger.getLogger(WordNetFrontEnd.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
}
