/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package newsAnalyzer.dataModel;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.EnumMap;
import java.util.Iterator;
import java.util.List;
import java.util.TreeSet;
import newsAnalyzer.dataModel.containers.SortedListFactory;
import newsAnalyzer.utils.TextUtils;

/**
 *
 * @author rcastari
 */
public class Article {

    protected Integer id = null;
    protected Date date = null;
    protected String title = null;
    protected String url = null;
    protected ArticleSource source;
    protected Boolean hasBeenAnalyzed = false;

    protected List<Author> authors = new ArrayList<Author>();
    protected EnumMap<Facet, List<TagDraft>> draftTagsMap = new EnumMap<Facet, List<TagDraft>>(Facet.class);
    protected EnumMap<Facet, TreeSet<NormalizedTag>> normalizedTagsMap = new EnumMap<Facet, TreeSet<NormalizedTag>>(Facet.class);
    
    protected static final SimpleDateFormat sdf = new SimpleDateFormat("dd-MM-yyyy");

    public Article(){

        Facet[] fl = Facet.values();
        for(int i = 0; i < fl.length ; i++){
            normalizedTagsMap.put(fl[i], SortedListFactory.getNormalizedTagSortedList(SortedListFactory.SORTED_BY_ID));
            draftTagsMap.put(fl[i], new ArrayList<TagDraft>());
        }

    }

    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public Date getDate() {
        return date;
    }

    public void setDate(Date date) {
        this.date = date;
    }

    public Integer getId() {
        return id;
    }

    public void setId(int id) {
        this.id = id;
    }

    public String getUrl() {
        return url;
    }

    public void setUrl(String url) {
        this.url = url;
    }

    public List<Author> getAuthors() {
        return authors;
    }

    public void addAuthor(Author author) {
        this.authors.add(author);
    }

    public void setHasBeenAnalyzed(boolean hasItBeen){
        this.hasBeenAnalyzed=hasItBeen;
    }

    public Boolean hasBeenAnalyzed(){
        return hasBeenAnalyzed;
    }

    public EnumMap<Facet, List<TagDraft>> getTagDrafts() {
        return draftTagsMap;
    }

    public List<TagDraft> getTagDraftsList(){
        List<TagDraft> l = new ArrayList<TagDraft>();
        Iterator<Facet> tdfi = draftTagsMap.keySet().iterator();
        while(tdfi.hasNext()){
            l.addAll(draftTagsMap.get(tdfi.next()));
        }
        return l;
    }

    public List<TagDraft> getTagDrafts(Facet facet){
        List<TagDraft> l = draftTagsMap.get(facet);
        if(l==null){
            l = new ArrayList<TagDraft>();
            draftTagsMap.put(facet, l);
        }
        return l;
    }

    public void addTagDraft(TagDraft td){

        List<TagDraft> tdl = draftTagsMap.get(td.getFacet());
        if(tdl == null){
            tdl = new ArrayList<TagDraft>();
            draftTagsMap.put(td.getFacet(), tdl);
        }
        tdl.add(td);

    }

    public void addTagDraftList(List<TagDraft> tdl) {

        Iterator<TagDraft> tdi = tdl.iterator();

        while (tdi.hasNext()) {
            addTagDraft(tdi.next());
        }

    }

    public TreeSet<NormalizedTag> getNormalizedTags(Facet facet){
        return normalizedTagsMap.get(facet);
    }

    public EnumMap<Facet, TreeSet<NormalizedTag>> getNormalizedTags(){
        return normalizedTagsMap;
    }

    /*

    public float distance(Article b){



        float d = 0f;

        List<Float> distances = new ArrayList<Float>();
        
        distances.add(MyDistance.multivaluedAttributeDistance(this.des_facet, b.getDes_facet()));
        distances.add(MyDistance.multivaluedAttributeDistance(this.geo_facet, b.getGeo_facet()));
        distances.add(MyDistance.multivaluedAttributeDistance(this.per_facet, b.getPer_facet()));
        distances.add(MyDistance.multivaluedAttributeDistance(this.org_facet, b.getOrg_facet()));

        Iterator<Float> i = distances.iterator();
        while(i.hasNext()){
            d+=i.next();
        }
        
        d=distances.isEmpty()?0:d/distances.size();

        return d;


    }
     *
     */

    @Override
    public String toString(){
        return toString(0);
    }

    public String toString(int tabs){
        String s = "";

        s += TextUtils.insertBlanks(tabs) + "Article {";
        s += System.getProperty("line.separator");

        tabs++;

        s += TextUtils.insertBlanks(tabs) + "id: ";
        if(id!=null){
            s += id;
        }else{
            s += "{}";
        }
        s += System.getProperty("line.separator");
        
        s += TextUtils.insertBlanks(tabs) + "Title: ";
        if(title!=null){
            s += title;
        }else{
            s += "{}";
        }
        s += System.getProperty("line.separator");

        s += TextUtils.insertBlanks(tabs) + "Source: ";
        if(source!=null){
            s += source;
        }else{
            s += "{}";
        }
        s += System.getProperty("line.separator");

        s += TextUtils.insertBlanks(tabs) + "Authors:";
        s += System.getProperty("line.separator");

        tabs++;

        Iterator<Author> authorIterator = authors.iterator();
        if (authorIterator.hasNext()) {
            while (authorIterator.hasNext()) {
                s += TextUtils.insertBlanks(tabs) + authorIterator.next().getName();
                s += System.getProperty("line.separator");
            }
        }else{
            s += "{}";
            s += System.getProperty("line.separator");
        }

        tabs--;
        
        s += TextUtils.insertBlanks(tabs) + "Date: ";
        if(date!=null){
            s+=sdf.format(date);
        }else{
            s += "{}";
        }
        s += System.getProperty("line.separator");
        
        s += TextUtils.insertBlanks(tabs) + "URL: ";
        if(url!=null){
            s+=url;
        }else{
            s += "{}";
        }
        s += System.getProperty("line.separator");

        s += TextUtils.insertBlanks(tabs) + "Tag drafts: ";
        s += System.getProperty("line.separator");
        
        tabs++;

        Iterator<Facet> itdf = draftTagsMap.keySet().iterator();
        if (itdf.hasNext()) {
            while (itdf.hasNext()) {
                Facet tdf = itdf.next();
                s += TextUtils.insertBlanks(tabs) + "Facet: " + tdf.name();
                s += System.getProperty("line.separator");

                tabs++;

                Iterator<TagDraft> itd = draftTagsMap.get(tdf).iterator();
                while (itd.hasNext()) {
                    s += itd.next().toString(tabs);
                    s += System.getProperty("line.separator");
                }

                tabs--;

            }
        }else{
            s += TextUtils.insertBlanks(tabs) + "{}";
        }

        tabs--;

        s += TextUtils.insertBlanks(tabs) + "Normalized Tags: ";
        s += System.getProperty("line.separator");

        tabs++;

        Iterator<Facet> fi = this.normalizedTagsMap.keySet().iterator();
        if (fi.hasNext()) {
            while (fi.hasNext()) {
                Facet f = fi.next();
                s += TextUtils.insertBlanks(tabs) + "Facet: " + f;
                s += System.getProperty("line.separator");

                tabs++;

                Iterator<NormalizedTag> nti = normalizedTagsMap.get(f).iterator();
                while (nti.hasNext()) {
                    s += nti.next().toString(tabs);
                    s += System.getProperty("line.separator");
                }

                tabs--;

            }
        }else{
            s += TextUtils.insertBlanks(tabs) + "{}";
        }
        tabs--;

        tabs--;

        s += TextUtils.insertBlanks(tabs) + "}";


        return s;
        
    }

    public ArticleSource getSource() {
        return source;
    }

    public void setSource(ArticleSource source) {
        this.source = source;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Article other = (Article) obj;
        return other.getId()==getId();
    }

    @Override
    public int hashCode() {
        int hash = 3;
        hash = 67 * hash + (this.id != null ? this.id.hashCode() : 0);
        return hash;
    }

}
