/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package newsAnalyzer.dataModel.DAOs;

import java.sql.Connection;
import newsAnalyzer.NewsAnalyzerProperties;
import newsAnalyzer.dataModel.DAOs.MySQL.MySQLDAOFactory;

/**
 *
 * @author rcastari
 */
public abstract class DAOFactory {
    // List of DAO types supported by the factory

    public static final int MySQL = 1;

    // There will be a method for each DAO that can be
    // created. The concrete factories will have to
    // implement these methods.
    public abstract ArticleDAO getArticleDAO();
    public abstract NormalizedTagDAO getNormalizedTagDAO();
    public abstract AuthorDAO getAuthorDAO();
    public abstract TagDraftDAO getTagDraftDAO();
    public abstract ArticleSourceDAO getArticleSourceDAO();
    public abstract Connection getConnection();
    public abstract ResultsExplorerDAO getResultsExplorerDAO();

    protected static DAOFactory getDAOFactory(
            int whichFactory) {

        switch (whichFactory) {
            case MySQL:
            default:
                return new MySQLDAOFactory();
        }
    }

    public static DAOFactory getDAOFactory(){
        String DBEngine = NewsAnalyzerProperties.getInstance().getProperty("DBEngine");
        if(DBEngine.equalsIgnoreCase("MySQL")){
            return getDAOFactory(MySQL);
        }
        return getDAOFactory(MySQL);
    }
}
