/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package newsAnalyzer.dataModel.DAOs.MySQL;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.TreeSet;
import newsAnalyzer.NewsAnalyzerProperties;
import newsAnalyzer.dataModel.Article;
import newsAnalyzer.dataModel.Author;
import newsAnalyzer.dataModel.DAOs.ArticleDAO;
import newsAnalyzer.dataModel.DAOs.DAOFactory;
import newsAnalyzer.dataModel.ArticleSource;
import newsAnalyzer.dataModel.DAOs.searchCriteriaObjects.ArticleCriteria;
import newsAnalyzer.dataModel.DAOs.searchCriteriaObjects.ArticleSourceCriteria;
import newsAnalyzer.dataModel.DAOs.searchCriteriaObjects.SearchCriteria;
import newsAnalyzer.dataModel.NormalizedTag;
import newsAnalyzer.dataModel.TagDraft;

/**
 *
 * @author rcastari
 */
public class MySQLArticleDAO extends MySQLGenericDAO<Article> implements ArticleDAO {

    //INSERT Parameters

    @Override
    protected boolean returnGeneratedKeys() {
        return true;
    }

    @Override
    protected String getInsertSQLStatement() {
        return "INSERT INTO Article(articleTitle, articleDate, articleUrl, ArticleSourceId, analyzed) VALUES(?, ?, ?, ?, ?)";
    }

    @Override
    protected void fillInsertPreparedStatementPreviousActions(Article a, Connection conn) throws SQLException {
        if(a.getSource()!=null){
            if (a.getSource().getId()==null) {
                a.setSource(DAOFactory.getDAOFactory().getArticleSourceDAO().load(a.getSource(),conn));
            }
        }

    }
    
    @Override
    protected void fillInsertPreparedStatement(PreparedStatement ps, Article a) throws SQLException {
        ps.setString(1, a.getTitle());
        ps.setDate(2, new java.sql.Date(a.getDate().getTime()));
        if (a.getUrl() == null) {
            ps.setNull(3, java.sql.Types.VARCHAR);
        } else if (a.getUrl().equals("")) {
            ps.setNull(3, java.sql.Types.VARCHAR);
        } else {
            ps.setString(3, a.getUrl());
        }
        if (a.getSource() == null) {
            ps.setNull(3, java.sql.Types.INTEGER);
        } else if (a.getSource().getId()==null) {
            ps.setNull(4, java.sql.Types.INTEGER);
        } else {
            ps.setInt(4, a.getSource().getId());
        }
        if(a.hasBeenAnalyzed()){
            ps.setInt(5, 1);
        }else{
            ps.setInt(5, 0);
        }
    }

    @Override
    protected void executeInsertPreviousActions(Article e, Connection conn) throws SQLException {
        //Do nothing
    }

    @Override
    protected Article afterInsertActions(Article a, Connection conn) throws SQLException {
        return a;
    }

    @Override
    protected Article afterInsertActions(Article a, int generatedKey, Connection conn) throws SQLException {

        a.setId(generatedKey);

        List<Author> al = a.getAuthors();

        for(int n=0; n < al.size(); n++){
            Author author = al.get(n);
            if(author.getId()==null){
                author = DAOFactory.getDAOFactory().getAuthorDAO().load(author,conn);
                al.set(n, author);
            }
            associateAuthor(a, author, conn);
        }

        Collection<List<TagDraft>> tdc = a.getTagDrafts().values();

        Iterator<List<TagDraft>> tdli = tdc.iterator();
        
        while (tdli.hasNext()) {

            List<TagDraft> tdl = tdli.next();

            for(int n=0; n<tdl.size(); n++){
                TagDraft td = tdl.get(n);
                if(td.getId()==null){
                    td = DAOFactory.getDAOFactory().getTagDraftDAO().load(td,conn);
                    tdl.set(n, td);
                }
                associateTagDraft(a, td, conn);
            }

        }

        Collection<TreeSet<NormalizedTag>> ntl = a.getNormalizedTags().values();

        Iterator<TreeSet<NormalizedTag>> ntli = ntl.iterator();

        while(ntli.hasNext()){
            List<NormalizedTag> listOfNewNTs = new ArrayList<NormalizedTag>();
            List<NormalizedTag> listOfOldNTs = new ArrayList<NormalizedTag>();
            TreeSet<NormalizedTag> ntts = ntli.next();
            Iterator<NormalizedTag> nti = ntts.iterator();
            while(nti.hasNext()){

                NormalizedTag nt = nti.next();
                NormalizedTag newNT = nt;

                if(nt.getId()==null){
                    newNT = DAOFactory.getDAOFactory().getNormalizedTagDAO().load(nt,conn);
                    listOfOldNTs.add(nt);
                    listOfNewNTs.add(newNT);
                }

                this.associateNormalizedTag(a, newNT, conn);

            }
            for(int n=0; n<listOfOldNTs.size(); n++){
                ntts.remove(listOfOldNTs.get(n));
                ntts.add(listOfNewNTs.get(n));
            }
        }

        return a;
    }

    @Override
    protected String getUpdateSQLStatement() {
        return "UPDATE Article "
                + "SET ArticleTitle = ?, articleDate = ?, ArticleSourceId = ?, articleUrl = ?, analyzed = ? "
                + "WHERE ArticleId = ?";
    }

    @Override
    protected boolean hasToBeUpdated(Article new_e, Article old_e) {
        return false;
    }

    @Override
    protected void fillUpdatePreparedStatementPreviousActions(Article e, Connection conn) throws SQLException {
        //Do nothing
    }

    @Override
    protected void fillUpdatePreparedStatement(PreparedStatement ps, Article a) throws SQLException {
        int id;
        try{
            id = a.getId();
            ps.setInt(6, id);
        }catch(NullPointerException ex){
            throw new SQLException("Trying to update a Article without providing its Id");
        }
        if(a.getTitle()!=null){
            ps.setString(1, a.getTitle());
        }else{
            throw new SQLException("Trying to update a Article with a null title");
        }
        if(a.getDate()!=null){
            ps.setDate(2, new java.sql.Date(a.getDate().getTime()));
        }else{
            throw new SQLException("Trying to update a Article with a null date");
        }
        if(a.getSource()!=null){
            if(a.getSource().getId()!=null){
                ps.setInt(3, a.getSource().getId());
            }else{
                throw new SQLException("Trying to update a Article with a source whith a null id");
            }
        }else{
            throw new SQLException("Trying to update a Article with a null source");
        }
        if(a.getUrl()!=null){
            ps.setString(4, a.getUrl());
        }else{
            ps.setNull(4, Types.VARCHAR);
        }
        if(a.hasBeenAnalyzed()){
            ps.setInt(5, 1);
        }else{
            ps.setInt(5, 0);
        }
    }

    @Override
    protected void executeUpdatePreviousActions(Article e, Connection conn) throws SQLException {
        //Do nothing
    }

    @Override
    protected Article afterUpdateActions(Article e, Connection conn) throws SQLException {

        //TODO Update authors

        //TODO Update TagDrafts

        //TODO Update NormalizedTags

        return e;
    }

    //SELECT Parameters

    @Override
    protected String getSelectSQLStatement(SearchCriteria<Article> a) throws SQLException {
        String sql = "SELECT articleId, articleTitle, articleDate, articleUrl, ArticleSourceId FROM Article WHERE 1=1";

        if (a != null) {
            if (a.getCriteria() != null) {
                if (a.getCriteria().getId() != null) {
                    sql += " AND articleId = ?";
                } else {
                    if (a.getCriteria().getTitle() != null) {
                        sql += " AND articleTitle = ?";
                    }
                    if (a.getCriteria().getDate() != null) {
                        sql += " AND articleDate = ?";
                    }
                    if (a.getCriteria().getUrl() != null) {
                        sql += " AND articleUrl = ?";
                    } else if (a.getNullDiscriminatorCriteria(ArticleCriteria.URL) == true) {
                        sql += " AND articleUrl IS NULL";
                    }
                    if (a.getCriteria().getSource() != null) {
                        sql += " AND ArticleSourceId = ?";
                    }
                    if(a.getCriteria().hasBeenAnalyzed()!=null){
                        sql += " AND analyzed = ?";
                    }
                }
            }
        }
        return sql;
    }

    @Override
    protected void fillSelectPreparedStatementPreviousActions(SearchCriteria<Article> e, Connection conn) throws SQLException {
        //Do nothing
    }

    @Override
    protected void fillSelectPreparedStatement(PreparedStatement ps, SearchCriteria<Article> a) throws SQLException {
        int n = 1;
        if (a != null) {
            if (a.getCriteria() != null) {
                if (a.getCriteria().getId() != null) {
                    ps.setInt(n, a.getCriteria().getId());
                    n++;
                } else {
                    if (a.getCriteria().getTitle() != null) {
                        ps.setString(n, a.getCriteria().getTitle());
                        n++;
                    }
                    if (a.getCriteria().getDate() != null) {
                        ps.setDate(n, new java.sql.Date(a.getCriteria().getDate().getTime()));
                        n++;
                    }
                    if (a.getCriteria().getUrl() != null) {
                        if (!a.getCriteria().getUrl().equals("")) {
                            ps.setString(n, a.getCriteria().getUrl());
                        }
                        n++;
                    }
                    if (a.getCriteria().getSource() != null) {
                        if (a.getCriteria().getSource().getId() != null) {
                            ps.setInt(n, a.getCriteria().getSource().getId());
                        } else {
                            ps.setNull(n, Types.INTEGER);
                        }
                        n++;
                    }
                    if (a.getCriteria().hasBeenAnalyzed() != null) {
                        if (a.getCriteria().hasBeenAnalyzed()) {
                            ps.setInt(n, 1);
                        } else {
                            ps.setInt(n, 0);
                        }
                        n++;
                    }
                }
            }
        }
    }

    @Override
    protected void executeSelectPreviousActions(SearchCriteria<Article> a, Connection conn) throws SQLException {
        //Do nothing
    }

    @Override
    protected List<Article> afterSelectActions(ResultSet rs, SearchCriteria<Article> a, Connection conn) throws SQLException {

        List<Article> list = new ArrayList<Article>();

        while(rs.next()){
            Article article = new Article();
            article.setId(rs.getInt(1));
            article.setTitle(rs.getString(2));
            article.setDate(rs.getDate(3));
            String url = rs.getString(4);
            if(!rs.wasNull()){
                article.setUrl(url);
            }
            ArticleSource as = new ArticleSource();
            as.setId(rs.getInt(5));

            List<ArticleSource> asl = (new MySQLArticleSourceDAO()).find((SearchCriteria<ArticleSource>)SearchCriteria.getSearchCriteria(as), conn);

            if(asl.size()!=1){
                throw new SQLException("Impossible to load ArticleSource with id="+as.getId());
            }

            as = asl.get(0);

            article.setSource(as);

            //load authors

            article.getAuthors().addAll(getAuthors(article, conn));

            //load tag' drafts

            article.addTagDraftList(getDraftTags(article, conn));
            
            //load normalized tags

            Iterator<NormalizedTag> nti = getNormalizedTags(article, conn).iterator();

            while(nti.hasNext()){
                NormalizedTag nt = nti.next();
                article.getNormalizedTags(nt.getFacet()).add(nt);
            }

            list.add(article);

        }

        return list;

    }

    public void associateAuthor(Article article, Author author) throws SQLException {
        Connection conn = MySQLDAOFactory.createConnection();

        associateAuthor(article, author, conn);

        if(conn!=null){
            try{
                conn.close();
            }catch(SQLException ex){
                //Cant' do nothing
            }
        }
    }

    public void associateAuthor(Article article, Author author, Connection conn) throws SQLException {

        PreparedStatement ps = conn.prepareStatement(""
                + "INSERT INTO ArticleHasAuthor (articleId, authorId) VALUES (?, ?)");

        if(author == null || article == null){
            throw new SQLException("Trying to insert null object!");
        }else if(author.getId() == null || article.getId()== null){
            throw new SQLException("Trying to insert object with null id!");
        }

        ps.setInt(1, article.getId());
        ps.setInt(2, author.getId());

        if (ps.executeUpdate() != 1) {
            throw new SQLException("Unable to insert Author-Article relation into the database!");
        }

        if (ps != null) {
            ps.close();
        }

    }

    public void associateTagDraft(Article a, TagDraft td) throws SQLException {
        Connection conn = MySQLDAOFactory.createConnection();

        associateTagDraft(a, td, conn);

        if(conn!=null){
            try{
                conn.close();
            }catch(SQLException ex){
                //Cant' do nothing
            }
        }
    }

    public void associateTagDraft(Article a, TagDraft td, Connection conn) throws SQLException {
        PreparedStatement ps = conn.prepareStatement(""
                + "INSERT INTO ArticleHasTagDraft (articleId, tagDraftId) VALUES (?, ?)");

        if(a == null || td == null){
            throw new SQLException("Trying to insert null object!");
        }else if(a.getId() == null || td.getId()== null){
            throw new SQLException("Trying to insert object with null id!");
        }

        ps.setInt(1, a.getId());
        ps.setInt(2, td.getId());

        if (ps.executeUpdate() != 1) {
            throw new SQLException("Unable to insert Author-TagDraft relation into the database!");
        }

        if (ps != null) {
            ps.close();
        }
    }

    public void associateNormalizedTag(Article a, NormalizedTag nt) throws SQLException {
        Connection conn = MySQLDAOFactory.createConnection();

        this.associateNormalizedTag(a, nt, conn);

        if(conn!=null){
            try{
                conn.close();
            }catch(SQLException ex){
                //Cant' do nothing
            }
        }
    }

    public void associateNormalizedTag(Article a, NormalizedTag nt, Connection conn) throws SQLException {
        PreparedStatement ps = conn.prepareStatement(""
                + "INSERT INTO ArticleHasNormalizedTag (articleId, normalizedTagId) VALUES (?, ?)");

        if(a == null || nt == null){
            throw new SQLException("Trying to insert null object!");
        }else if(a.getId() == null || nt.getId()== null){
            throw new SQLException("Trying to insert object with null id!");
        }

        ps.setInt(1, a.getId());
        ps.setInt(2, nt.getId());

        if (ps.executeUpdate() != 1) {
            throw new SQLException("Unable to insert Author-NormalizedTag relation into the database!");
        }

        if (ps != null) {
            ps.close();
        }
    }

    private List<Author> getAuthors(Article a, Connection conn) throws SQLException {

        List<Author> list = new ArrayList<Author>();

        String sql = "SELECT authorId FROM ArticleHasAuthor WHERE articleId = ?";

        PreparedStatement ps = conn.prepareStatement(sql);

        ps.setInt(1, a.getId());

        ResultSet rs = ps.executeQuery();

        while(rs.next()){
            Author author = new Author();
            author.setId(rs.getInt(1));
            List<Author> al = (new MySQLAuthorDAO()).find((SearchCriteria<Author>)SearchCriteria.getSearchCriteria(author), conn);
            if(al.size()!=1){
                throw new SQLException("Unable to load Author with id="+author.getId());
            }
            author = al.get(0);
            list.add(author);
        }

        return list;

    }

    private List<TagDraft> getDraftTags(Article a, Connection conn) throws SQLException {

        List<TagDraft> list = new ArrayList<TagDraft>();

        String sql = "SELECT tagDraftId FROM ArticleHasTagDraft WHERE articleId = ?";

        PreparedStatement ps = conn.prepareStatement(sql);

        ps.setInt(1, a.getId());

        ResultSet rs = ps.executeQuery();

        while(rs.next()){
            TagDraft td = new TagDraft();
            td.setId(rs.getInt(1));
            List<TagDraft> tdl = (new MySQLTagDraftDAO()).find((SearchCriteria<TagDraft>)SearchCriteria.getSearchCriteria(td), conn);
            if(tdl.size()!=1){
                throw new SQLException("Unable to load TagDraft with id="+td.getId());
            }
            td = tdl.get(0);
            list.add(td);
        }

        return list;

    }

    private List<NormalizedTag> getNormalizedTags(Article a, Connection conn) throws SQLException {

        List<NormalizedTag> list = new ArrayList<NormalizedTag>();

        String sql = "SELECT normalizedTagId FROM ArticleHasNormalizedTag WHERE articleId = ?";

        PreparedStatement ps = conn.prepareStatement(sql);

        ps.setInt(1, a.getId());

        ResultSet rs = ps.executeQuery();

        while(rs.next()){
            NormalizedTag nt = new NormalizedTag();
            nt.setId(rs.getInt(1));
            List<NormalizedTag> ntl = (new MySQLNormalizedTagDAO()).find((SearchCriteria<NormalizedTag>)SearchCriteria.getSearchCriteria(nt), conn);
            if(ntl.size()!=1){
                throw new SQLException("Unable to load NormalizedTag with id="+nt.getId());
            }
            nt = ntl.get(0);
            list.add(nt);
        }

        return list;

    }

    public int getArticleCount(Date date, ArticleSource as) throws SQLException {

        int count = 0;

        Connection conn = MySQLDAOFactory.createConnection();

        String sql = "SELECT COUNT(*) FROM Article WHERE articleDate = ? ";

        if(as!=null){
            sql += "AND articleSourceId = ?";
        }

        PreparedStatement ps = conn.prepareStatement(sql);

        ps.setDate(1, new java.sql.Date(date.getTime()));
        if (as != null) {
            if (as.getId() != null) {
                ArticleSourceCriteria asc = new ArticleSourceCriteria(as);
                List<ArticleSource> asl = DAOFactory.getDAOFactory().getArticleSourceDAO().find(asc, conn);
                if (asl.isEmpty()) {
                    throw new SQLException("Article Source not found!");
                } else if (asl.size() > 1) {
                    throw new SQLException("Ambiguous ArticleSource");
                } else {
                    as = asl.get(0);
                }
            }

            ps.setInt(2, as.getId());

        }

        ResultSet rs = ps.executeQuery();

        while(rs.next()){
            count = rs.getInt(1);
        }

        if(conn!=null){
            try{
                conn.close();
            }catch (SQLException ex){
                //There's nothing we can do
            }
        }

        return count;
    }

    protected void preLoadActions(Article a, Connection conn) throws SQLException {
        if(a.getSource()!=null){
            if(a.getSource().getId()==null){
                a.setSource(DAOFactory.getDAOFactory().getArticleSourceDAO().load(a.getSource(), conn));
            }
        }

        Iterator<List<TagDraft>> tdli = a.getTagDrafts().values().iterator();

        MySQLTagDraftDAO tdDAO = null;

        if(tdli.hasNext()){
            tdDAO = new MySQLTagDraftDAO();
        }

        while(tdli.hasNext()){
            List<TagDraft> tdl = tdli.next();

            for(int i=0; i<tdl.size(); i++){
                if(tdl.get(i).getId()!=null){
                    tdl.set(i, tdDAO.load(tdl.get(i), conn));
                }
            }
        }
        
        //MySQLNormalizedTagDAO ntDAO = null;
        //TODO ampliar el mètode als normalized tags

    }

    public void setHasBeenAnalyzed(Article a) throws SQLException {
        Connection conn = MySQLDAOFactory.createConnection();

        this.setHasBeenAnalyzed(a, conn);

        if(conn != null){
            conn.close();
        }
    }

    public void setHasBeenAnalyzed(Article a, Connection conn) throws SQLException {
        String sql = ""
                + "UPDATE Article "
                + "SET analyzed = 1 "
                + "WHERE articleId = ?";

        PreparedStatement ps = conn.prepareStatement(sql);

        ps.setInt(1, a.getId());

        if(NewsAnalyzerProperties.getInstance().debug()){
            System.out.println(ps);
        }

        int n = ps.executeUpdate();

        if(n==0){
            throw new SQLException("Error updating Article with id = "+a.getId());
        }

        if(ps!=null){
            ps.close();
        }
    }

}
