/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package newsAnalyzer.dataModel.DAOs.MySQL;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import newsAnalyzer.dataModel.Author;
import newsAnalyzer.dataModel.DAOs.AuthorDAO;
import newsAnalyzer.dataModel.DAOs.searchCriteriaObjects.SearchCriteria;

/**
 *
 * @author rcastari
 */
public class MySQLAuthorDAO extends MySQLGenericDAO<Author> implements AuthorDAO {

    //INSERT Parameters

    @Override
    protected boolean returnGeneratedKeys() {
        return true;
    }

    @Override
    protected String getInsertSQLStatement() {
        return "INSERT INTO Author (authorName) VALUES (?)";
    }

    @Override
    protected void fillInsertPreparedStatementPreviousActions(Author a, Connection conn) throws SQLException {
        //Do nothing
    }
    
    @Override
    protected void fillInsertPreparedStatement(PreparedStatement ps, Author a) throws SQLException {
        ps.setString(1, a.getName());
    }

    @Override
    protected void executeInsertPreviousActions(Author a, Connection conn) throws SQLException {
        //Do nothing
    }

    @Override
    protected Author afterInsertActions(Author a, Connection conn) throws SQLException {
        return a;
    }

    @Override
    protected Author afterInsertActions(Author a, int generatedKey, Connection conn) throws SQLException {
        a.setId(generatedKey);
        return a;
    }

    @Override
    protected String getUpdateSQLStatement() {
        return "UPDATE Author "
                + "SET authorName = ? "
                + "WHERE AuthorId = ?";
    }

    @Override
    protected boolean hasToBeUpdated(Author new_e, Author old_e) {
        return false;
    }



    @Override
    protected void fillUpdatePreparedStatementPreviousActions(Author e, Connection conn) throws SQLException {
        //Do nothing
    }

    @Override
    protected void fillUpdatePreparedStatement(PreparedStatement ps, Author a) throws SQLException {
        int id;
        try{
            id = a.getId();
            ps.setInt(2, id);
        }catch(NullPointerException ex){
            throw new SQLException("Trying to update an Author without providing its Id");
        }
        if(a.getName()!=null){
            ps.setString(1, a.getName());
        }else{
            throw new SQLException("Trying to update an Author with a null name");
        }
    }

    @Override
    protected void executeUpdatePreviousActions(Author e, Connection conn) throws SQLException {
        //Do nothing
    }

    @Override
    protected Author afterUpdateActions(Author e, Connection conn) throws SQLException {
        return e;
    }

    //SELECT Parameters

    @Override
    protected String getSelectSQLStatement(SearchCriteria<Author> a) throws SQLException {
        String sql = "SELECT authorId, authorName FROM Author WHERE 1=1";
        if (a != null) {
            if (a.getCriteria().getId() != null) {
                sql += " AND authorId = ?";
            } else {
                if (a.getCriteria().getName() != null) {
                    sql += " AND authorName = ?";
                }
            }

        }
        return sql;
    }

    @Override
    protected void fillSelectPreparedStatementPreviousActions(SearchCriteria<Author> a, Connection conn) throws SQLException {
        //Do nothing
    }

    @Override
    protected void fillSelectPreparedStatement(PreparedStatement ps, SearchCriteria<Author> a) throws SQLException {
        int n=1;
        if(a.getCriteria().getId()!=null){
            ps.setInt(n, a.getCriteria().getId());
            n++;
        } else {
            if (a.getCriteria().getName() != null) {
                ps.setString(n, a.getCriteria().getName());
                n++;
            }
        }
    }

    @Override
    protected void executeSelectPreviousActions(SearchCriteria<Author> a, Connection conn) throws SQLException {
        //Do nothing
    }
    
    @Override
    protected List<Author> afterSelectActions(ResultSet rs, SearchCriteria<Author> a, Connection conn) throws SQLException {

        List<Author> list = new ArrayList<Author>();

        while(rs.next()){

            Author author = new Author();

            author.setId(rs.getInt(1));
            author.setName(rs.getString(2));

            list.add(author);
        }

        return list;

    }

    @Override
    protected void preLoadActions(Author e, Connection conn) throws SQLException {
        //Do nothing
    }



}
