/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package newsAnalyzer.dataModel.DAOs.MySQL;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.logging.Level;
import java.util.logging.Logger;
import newsAnalyzer.NewsAnalyzerProperties;
import newsAnalyzer.dataModel.DAOs.ResultsExplorerDAO;
import newsAnalyzer.dataModel.DAOs.ArticleDAO;
import newsAnalyzer.dataModel.DAOs.AuthorDAO;
import newsAnalyzer.dataModel.DAOs.DAOFactory;
import newsAnalyzer.dataModel.DAOs.ArticleSourceDAO;
import newsAnalyzer.dataModel.DAOs.NormalizedTagDAO;
import newsAnalyzer.dataModel.DAOs.TagDraftDAO;

/**
 *
 * @author rcastari
 */
public class MySQLDAOFactory extends DAOFactory {

    protected static final String DRIVER = "com.mysql.jdbc.Driver";
    protected static String DBURL = null;

    public MySQLDAOFactory() {

        NewsAnalyzerProperties connProperties = NewsAnalyzerProperties.getInstance();
        
        DBURL = "jdbc:mysql://" + connProperties.getProperty("DBHost")
                + ":" + connProperties.getProperty("DBPort")
                + "/" + connProperties.getProperty("DBName")
                + "?user=" + connProperties.getProperty("DBUser")
                + "&password=" + connProperties.getProperty("DBPassword");
    }

    // method to create MySQL connections
    public static Connection createConnection() {

        // Use DRIVER and DBURL to create a connection

        try {

            Class.forName(DRIVER).newInstance();

        } catch (ClassNotFoundException ex) {
            Logger.getLogger(MySQLDAOFactory.class.getName()).log(Level.SEVERE, null, ex);
        } catch (InstantiationException ex) {
            Logger.getLogger(MySQLDAOFactory.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            Logger.getLogger(MySQLDAOFactory.class.getName()).log(Level.SEVERE, null, ex);
        }

        Connection conn = null;

        try {

            conn = DriverManager.getConnection(DBURL);

        } catch (SQLException ex) {
            // handle any errors
            System.out.println("SQLException: " + ex.getMessage());
            System.out.println("SQLState: " + ex.getSQLState());
            System.out.println("VendorError: " + ex.getErrorCode());
        }

        return conn;

    }

    public ArticleDAO getArticleDAO() {
        // MySQLArticleDAO implements ArticleDAO
        return new MySQLArticleDAO();
    }

    public TagDraftDAO getTagDraftDAO() {
        return new MySQLTagDraftDAO();
    }

    public AuthorDAO getAuthorDAO(){
        return new MySQLAuthorDAO();
    }

    public NormalizedTagDAO getNormalizedTagDAO(){
        return new MySQLNormalizedTagDAO();
    }
    
    @Override
    public ArticleSourceDAO getArticleSourceDAO(){
        return new MySQLArticleSourceDAO();
    }

    public Connection getConnection(){
        return createConnection();
    }

    @Override
    public ResultsExplorerDAO getResultsExplorerDAO() {
        return new MySQLResultsExplorerDAO();
    }


}
