/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package newsAnalyzer.dataModel.DAOs.MySQL;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import newsAnalyzer.NewsAnalyzerProperties;
import newsAnalyzer.dataModel.DAOs.GenericDAO;
import newsAnalyzer.dataModel.DAOs.searchCriteriaObjects.SearchCriteria;

/**
 *
 * @author rcastari
 */
public abstract class MySQLGenericDAO<E> implements GenericDAO<E> {

    protected abstract String getInsertSQLStatement();

    protected abstract String getUpdateSQLStatement();

    protected abstract boolean returnGeneratedKeys();

    protected abstract void fillInsertPreparedStatementPreviousActions(E e, Connection conn) throws SQLException;

    protected abstract void fillUpdatePreparedStatementPreviousActions(E e, Connection conn) throws SQLException;

    protected abstract void fillInsertPreparedStatement(PreparedStatement ps, E e) throws SQLException;

    protected abstract void fillUpdatePreparedStatement(PreparedStatement ps, E e) throws SQLException;

    protected abstract void executeInsertPreviousActions(E e, Connection conn) throws SQLException;

    protected abstract void executeUpdatePreviousActions(E e, Connection conn) throws SQLException;

    protected abstract E afterInsertActions(E e, Connection conn) throws SQLException;

    protected abstract E afterInsertActions(E e, int generatedKey, Connection conn) throws SQLException;

    protected abstract E afterUpdateActions(E e, Connection conn) throws SQLException;

    protected abstract boolean hasToBeUpdated(E new_e, E old_e);

    protected abstract String getSelectSQLStatement(SearchCriteria<E> esc) throws SQLException;

    protected abstract void fillSelectPreparedStatementPreviousActions(SearchCriteria<E> esc, Connection conn) throws SQLException;

    protected abstract void fillSelectPreparedStatement(PreparedStatement ps, SearchCriteria<E> esc) throws SQLException;

    protected abstract void executeSelectPreviousActions(SearchCriteria<E> esc, Connection conn) throws SQLException;

    protected abstract List<E> afterSelectActions(ResultSet rs, SearchCriteria<E> esc, Connection conn) throws SQLException;

    protected SearchCriteria<E> obtainLoadSearchCriteria(E e) {
        return (SearchCriteria<E>) SearchCriteria.getSearchCriteria(e);
    }

    protected abstract void preLoadActions(E e, Connection conn) throws SQLException;

    public E save(E e) throws SQLException {

        E re = null;

        Connection conn = MySQLDAOFactory.createConnection();

        re = save(e, conn);

        if (conn != null) {
            try {
                conn.close();
            } catch (SQLException ex) {
                // ignore
            }
        }

        return re;
    }

    public E save(E e, Connection conn) throws SQLException {

        E re = null;

        boolean update = false;
        boolean insert = false;
        boolean autocommit = conn.getAutoCommit();

        List<E> el = find(this.obtainLoadSearchCriteria(e), conn);

        PreparedStatement stmt = null;

        conn.setAutoCommit(false);

        if (!el.isEmpty()) {

            if (el.size() == 1) {
                if (this.hasToBeUpdated(e, el.get(0))) {
                    update = true;
                    insert = false;
                }
            } else {
                throw new SQLException("Trying to update ambiguous element");
            }
        } else {
            insert = true;
            update = false;
        }

        try {

            if (update) {
                stmt = conn.prepareStatement(getUpdateSQLStatement());

                fillUpdatePreparedStatementPreviousActions(e, conn);

                fillUpdatePreparedStatement(stmt, e);

                if (NewsAnalyzerProperties.getInstance().debug()) {
                    System.out.println(stmt);
                }

                executeUpdatePreviousActions(e, conn);

            } else if (insert){

                if (returnGeneratedKeys()) {
                    stmt = conn.prepareStatement(getInsertSQLStatement(),
                            Statement.RETURN_GENERATED_KEYS);
                } else {
                    stmt = conn.prepareStatement(getInsertSQLStatement());
                }

                fillInsertPreparedStatementPreviousActions(e, conn);

                fillInsertPreparedStatement(stmt, e);

                if (NewsAnalyzerProperties.getInstance().debug()) {
                    System.out.println(stmt);
                }

                executeInsertPreviousActions(e, conn);

            }

            if(insert||update){
                if (stmt.executeUpdate() != 1) {
                    throw new SQLException("Bad Update!");
                }
            }

            if (update) {

                re = afterUpdateActions(e, conn);

            } else if (insert){

                if (returnGeneratedKeys()) {

                    ResultSet rs = null;

                    int generatedKey;

                    rs = stmt.getGeneratedKeys();
                    if (rs.next()) {
                        generatedKey = rs.getInt(1);
                    } else {
                        throw new SQLException("No key was generated");
                    }

                    if (rs != null) {
                        try {
                            rs.close();
                        } catch (SQLException ex) {
                            // ignore
                        }
                    }

                    re = afterInsertActions(e, generatedKey, conn);

                } else {
                    re = afterInsertActions(e, conn);
                }

            }

            conn.commit();

        } catch (SQLException ex) {
            Logger.getLogger(MySQLGenericDAO.class.getName()).log(Level.SEVERE, null, ex);
            conn.rollback();
        } finally {

            if (stmt != null) {
                try {
                    stmt.close();
                } catch (SQLException ex) {
                    // ignore
                }
            }

            try {
                conn.setAutoCommit(autocommit);
            } catch (SQLException ex) {
                //ignore
            }
        }
        return re;
    }

    public List<E> find(SearchCriteria<E> esc) throws SQLException {

        List<E> list = new ArrayList<E>();

        Connection conn = MySQLDAOFactory.createConnection();

        list = find(esc, conn);

        if (conn != null) {
            try {
                conn.close();
            } catch (SQLException ex) {
                // ignore
            }
        }

        return list;

    }

    public List<E> find(SearchCriteria<E> esc, Connection conn) throws SQLException {

        List<E> list = new ArrayList<E>();

        PreparedStatement stmt = null;

        ResultSet rs = null;

        try {

            fillSelectPreparedStatementPreviousActions(esc, conn);

            stmt = conn.prepareStatement(getSelectSQLStatement(esc));

            fillSelectPreparedStatement(stmt, esc);

            if (NewsAnalyzerProperties.getInstance().debug()) {
                System.out.println(stmt);
            }

            executeSelectPreviousActions(esc, conn);

            rs = stmt.executeQuery();

            list = afterSelectActions(rs, esc, conn);

        } catch (SQLException ex) {
            Logger.getLogger(MySQLGenericDAO.class.getName()).log(Level.SEVERE, null, ex);

        } finally {

            if (rs != null) {
                try {
                    rs.close();
                } catch (SQLException ex) {
                    // ignore
                }
            }

            if (stmt != null) {
                try {
                    stmt.close();
                } catch (SQLException ex) {
                    // ignore
                }
            }
        }

        return list;

    }

    public List<E> findAll() throws SQLException {

        return find(null);

    }

    public List<E> findAll(Connection conn) throws SQLException {

        return find(null, conn);

    }

    public E load(E e) throws SQLException {

        Connection conn = MySQLDAOFactory.createConnection();

        E re = load(e, conn);

        if (conn != null) {
            try {
                conn.close();
            } catch (SQLException ex) {
                // ignore
            }
        }

        return re;
    }

    public E load(E e, Connection conn) throws SQLException {
        E re = null;

        List<E> el = null;

        preLoadActions(e, conn);

        try {
            el = find(obtainLoadSearchCriteria(e), conn);
        } catch (SQLException ex) {
            Logger.getLogger(MySQLGenericDAO.class.getName()).log(Level.SEVERE, null, ex);
        }
        if (el.isEmpty()) {
            re = save(e, conn);
        } else if (el.size() == 1) {
            re = el.get(0);
        } else {
            throw new SQLException("Found ambiguity on load operation");
        }
        return re;
    }
}
