/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package newsAnalyzer.dataModel.DAOs.MySQL;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.List;
import newsAnalyzer.dataModel.NormalizedTag;
import newsAnalyzer.dataModel.DAOs.NormalizedTagDAO;
import newsAnalyzer.dataModel.DAOs.searchCriteriaObjects.SearchCriteria;
import newsAnalyzer.dataModel.Facet;

/**
 *
 * @NormalizedTag rcastari
 */
public class MySQLNormalizedTagDAO extends MySQLGenericDAO<NormalizedTag> implements NormalizedTagDAO {

    //INSERT Parameters

    @Override
    protected boolean returnGeneratedKeys() {
        return true;
    }

    @Override
    protected String getInsertSQLStatement() {
        return "INSERT INTO NormalizedTag (NormalizedTagName, NormalizedTagFacetId) VALUES (?, ?)";
    }

    @Override
    protected void fillInsertPreparedStatementPreviousActions(NormalizedTag a, Connection conn) throws SQLException {
        //Do nothing
    }
    
    @Override
    protected void fillInsertPreparedStatement(PreparedStatement ps, NormalizedTag a) throws SQLException {
        ps.setString(1, a.getName());
        if (a.getFacet() != null){
            ps.setInt(2, a.getFacet().getId());
        }else{
            ps.setNull(2, Types.INTEGER);
        }
    }

    @Override
    protected void executeInsertPreviousActions(NormalizedTag a, Connection conn) throws SQLException {
        //Do nothing
    }

    @Override
    protected NormalizedTag afterInsertActions(NormalizedTag a, Connection conn) throws SQLException {
        return a;
    }

    @Override
    protected NormalizedTag afterInsertActions(NormalizedTag a, int generatedKey, Connection conn) throws SQLException {
        a.setId(generatedKey);
        return a;
    }

    @Override
    protected String getUpdateSQLStatement() {
        return "UPDATE NormalizedTag "
                + "SET NormalizedTagName = ?, NormalizedTagFacetId = ? "
                + "WHERE NormalizedTagId = ?";
    }

    @Override
    protected boolean hasToBeUpdated(NormalizedTag new_e, NormalizedTag old_e) {
        boolean result = false;
        return result;
    }



    @Override
    protected void fillUpdatePreparedStatementPreviousActions(NormalizedTag e, Connection conn) throws SQLException {
        //Do nothing
    }

    @Override
    protected void fillUpdatePreparedStatement(PreparedStatement ps, NormalizedTag a) throws SQLException {
        int id;
        try{
            id = a.getId();
            ps.setInt(3, id);
        }catch(NullPointerException ex){
            throw new SQLException("Trying to update a NormalizedTag without providing its Id");
        }
        if(a.getName()!=null){
            ps.setString(1, a.getName());
        }else{
            throw new SQLException("Trying to update a NormalizedTag with a null name");
        }
        if(a.getFacet()!=null){
            ps.setInt(4, a.getFacet().getId());
        }else{
            ps.setNull(4, Types.INTEGER);
        }
    }

    @Override
    protected void executeUpdatePreviousActions(NormalizedTag e, Connection conn) throws SQLException {
        //Do nothing
    }

    @Override
    protected NormalizedTag afterUpdateActions(NormalizedTag e, Connection conn) throws SQLException {
        return e;
    }

    //SELECT Parameters

    @Override
    protected String getSelectSQLStatement(SearchCriteria<NormalizedTag> a) throws SQLException {
        
        String sql ="SELECT NormalizedTagId, NormalizedTagName, NormalizedTagFacetId FROM NormalizedTag WHERE 1=1";
        if (a != null) {
            if (a.getCriteria().getId() != null) {
                sql += " AND NormalizedTagId = ?";
            } else {
                if (a.getCriteria().getName() != null) {
                    sql += " AND NormalizedTagName = ?";
                }
                if (a.getCriteria().getFacet() != null) {
                    sql += " AND NormalizedTagFacetId = ?";
                }
            }
        }
        return sql;
    }

    @Override
    protected void fillSelectPreparedStatementPreviousActions(SearchCriteria<NormalizedTag> a, Connection conn) throws SQLException {
        //Do nothing
    }

    @Override
    protected void fillSelectPreparedStatement(PreparedStatement ps, SearchCriteria<NormalizedTag> a) throws SQLException {
        int n=1;
        if(a.getCriteria().getId()!=null){
            ps.setInt(n, a.getCriteria().getId());
            n++;
        } else {
            if (a.getCriteria().getName() != null) {
                ps.setString(n, a.getCriteria().getName());
                n++;
            }
            if (a.getCriteria().getFacet() != null) {
                ps.setInt(n, a.getCriteria().getFacet().getId());
                n++;
            }
        }
    }

    @Override
    protected void executeSelectPreviousActions(SearchCriteria<NormalizedTag> a, Connection conn) throws SQLException {
        //Do nothing
    }
    
    @Override
    protected List<NormalizedTag> afterSelectActions(ResultSet rs, SearchCriteria<NormalizedTag> a, Connection conn) throws SQLException {

        List<NormalizedTag> list = new ArrayList<NormalizedTag>();

        while(rs.next()){

            NormalizedTag nt = new NormalizedTag();

            nt.setId(rs.getInt(1));
            nt.setName(rs.getString(2));
            nt.setFacet(Facet.getFacet(rs.getInt(3)));

            list.add(nt);
        }

        return list;

    }

    @Override
    protected SearchCriteria<NormalizedTag> obtainLoadSearchCriteria(NormalizedTag e) {
        NormalizedTag nt = new NormalizedTag();
        nt.setId(e.getId());
        nt.setName(e.getName());
        nt.setFacet(e.getFacet());
        SearchCriteria<NormalizedTag> sc = (SearchCriteria<NormalizedTag>) SearchCriteria.getSearchCriteria(nt);
        return sc;
    }

    @Override
    protected void preLoadActions(NormalizedTag e, Connection conn) throws SQLException {
        //Do nothing
    }



}
