/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package newsAnalyzer.dataModel.DAOs.MySQL;

import java.sql.PreparedStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import newsAnalyzer.NewsAnalyzerProperties;
import newsAnalyzer.dataModel.ArticleSource;
import newsAnalyzer.dataModel.DAOs.ResultsExplorerDAO;
import newsAnalyzer.dataModel.Facet;
import newsAnalyzer.dataModel.NormalizedTag;
import newsAnalyzer.dataModel.TagDraft;
import newsAnalyzer.dataModel.statistics.NormalizedTagStatistics;
import newsAnalyzer.retrievers.ArticleRetrieverModule;

/**
 *
 * @author rcastari
 */
public class MySQLResultsExplorerDAO implements ResultsExplorerDAO {

    public List<NormalizedTagStatistics> getNTxFontStatistics() throws Exception {

        return this.getNTxFontStatistics(null, null);

    }

    public List<NormalizedTagStatistics> getNTxFontStatistics(Date sinceDate, Date toDate) throws Exception {

        List<NormalizedTagStatistics> statistics = new ArrayList<NormalizedTagStatistics>();

        try {
            ArticleSource[] sources = (new ArticleRetrieverModule()).getArticleSources();

            Connection conn = MySQLDAOFactory.createConnection();

            String sqlQuery = ""
                    + "SELECT NormalizedTag.normalizedTagId, normalizedTagName, normalizedTagFacetId, COUNT(articleId) AS count "
                    + "FROM NormalizedTag NATURAL JOIN ArticleHasNormalizedTag ";

            if(sinceDate != null || toDate != null){
                sqlQuery += "NATURAL JOIN Article "
                            + "WHERE 1=1 ";
                if(sinceDate != null){
                    sqlQuery += "AND articleDate >= ? ";
                }
                if(toDate != null){
                    sqlQuery += "AND articleDate <= ? ";
                }
            }

            sqlQuery += "GROUP BY NormalizedTag.normalizedTagId";

            PreparedStatement ps = conn.prepareStatement(sqlQuery);

            if(sinceDate != null){
                ps.setDate(1, new java.sql.Date(sinceDate.getTime()));
            }

            if(toDate != null){
                if(sinceDate != null){
                    ps.setDate(2, new java.sql.Date(toDate.getTime()));
                }else{
                    ps.setDate(1, new java.sql.Date(toDate.getTime()));
                }
            }


            if(NewsAnalyzerProperties.getInstance().debug()){
                System.out.println(ps);
            }

            ResultSet rs = ps.executeQuery();

            String sqlQuery2 = "SELECT COUNT(articleId) FROM Article ";
            
            if(sinceDate != null || toDate != null){
                sqlQuery2 += "WHERE 1=1 ";
                if(sinceDate != null){
                    sqlQuery2 += "AND articleDate >= ? ";
                }
                if(toDate != null){
                    sqlQuery2 += "AND articleDate <= ? ";
                }
            }

            PreparedStatement ps2 = conn.prepareStatement(sqlQuery2);

            if(sinceDate != null){
                ps2.setDate(1, new java.sql.Date(sinceDate.getTime()));
            }

            if(toDate != null){
                if(sinceDate != null){
                    ps2.setDate(2, new java.sql.Date(toDate.getTime()));
                }else{
                    ps2.setDate(1, new java.sql.Date(toDate.getTime()));
                }
            }

            if(NewsAnalyzerProperties.getInstance().debug()){
                System.out.println(ps2);
            }

            ResultSet rs2 = ps2.executeQuery();

            int N = 0;

            if(rs2.next()){
                N = rs2.getInt(1);
            }else{
                throw new SQLException("No results for sql query: "+ps2);
            }

            if(ps2!=null){
                ps2.close();
            }

            if(rs2!=null){
                rs2.close();
            }

            sqlQuery = ""
                    + "SELECT COUNT(articleId) AS freq "
                    + "FROM "
                    + "ArticleHasNormalizedTag "
                    + "NATURAL JOIN "
                    + "Article "
                    + "WHERE normalizedTagId = ? AND articleSourceId = ? ";

            sqlQuery2 = ""
                    + "SELECT COUNT(articleId) FROM Article "
                    + "WHERE articleSourceId = ? ";

            if (sinceDate != null) {
                sqlQuery += "AND articleDate >= ? ";
                sqlQuery2 += "AND articleDate >= ? ";
            }
            
            if (toDate != null) {
                sqlQuery += "AND articleDate <= ? ";
                sqlQuery2 += "AND articleDate <= ? ";
            }

            while(rs.next()){

                int ntId = rs.getInt(1);
                String ntName = rs.getString(2);
                Facet ntFacet = Facet.getFacet(rs.getInt(3));
                int ntGlobalCount = ntGlobalCount = rs.getInt(4);
                float globalFreq = 0;
                if(ntGlobalCount!=0){
                    if(N!=0){
                        globalFreq = ntGlobalCount / (float) N;
                    }
                }

                NormalizedTagStatistics nts2 = new NormalizedTagStatistics(ntId, ntName, ntFacet, globalFreq);

                Integer count = null;

                for (int s = 0; s < sources.length; s++) {

                    PreparedStatement ps3 = conn.prepareStatement(sqlQuery);

                    Float freq = null;

                    ps3.setInt(1, ntId);
                    ps3.setInt(2, sources[s].getId());

                    if (sinceDate != null) {
                        ps3.setDate(3, new java.sql.Date(sinceDate.getTime()));
                    }

                    if (toDate != null) {
                        if (sinceDate != null) {
                            ps3.setDate(4, new java.sql.Date(toDate.getTime()));
                        } else {
                            ps3.setDate(3, new java.sql.Date(toDate.getTime()));
                        }
                    }

                    if (NewsAnalyzerProperties.getInstance().debug()) {
                        System.out.println(ps3);
                    }

                    ResultSet rs3 = ps3.executeQuery();

                    if(rs3.next()){

                        count = rs3.getInt(1);

                    }

                    if(count!=0){
                        PreparedStatement ps4 = conn.prepareStatement(sqlQuery2);

                        ps4.setInt(1, sources[s].getId());

                        if (sinceDate != null) {
                            ps4.setDate(2, new java.sql.Date(sinceDate.getTime()));
                        }

                        if (toDate != null) {
                            if (sinceDate != null) {
                                ps4.setDate(3, new java.sql.Date(toDate.getTime()));
                            } else {
                                ps4.setDate(2, new java.sql.Date(toDate.getTime()));
                            }
                        }

                        if (NewsAnalyzerProperties.getInstance().debug()) {
                            System.out.println(ps4);
                        }

                        ResultSet rs4 = ps4.executeQuery();

                        int ASN = 0;

                        if(rs4.next()){
                            ASN = rs4.getInt(1);
                        }else{
                            throw new SQLException("No results for sql query: "+ps4);
                        }

                        if(ASN!=0){
                            freq = count / (float)ASN;
                        }else{
                            count = 0;
                        }

                        if(ps4!=null){
                            ps4.close();
                        }

                        if(rs4!=null){
                            rs4.close();
                        }
                    }else{
                        freq = (float)0;
                    }

                    if(rs3 != null){
                        rs3.close();
                    }

                    if(ps3 != null){
                        ps3.close();
                    }

                    nts2.addArticlesxSourceCount(sources[s], freq);

                }

                statistics.add(nts2);

            }

            if(rs != null){
                rs.close();
            }

            if(ps != null){
                ps.close();
            }


        } catch (Exception ex) {
            Logger.getLogger(MySQLResultsExplorerDAO.class.getName()).log(Level.SEVERE, null, ex);
            throw ex;
        }

        return statistics;

    }

    public Float getNormalizationAccuracy() {

        Float accuracy = null;

        try {

            Connection conn = MySQLDAOFactory.createConnection();

            String sqlQuery = ""
                    + "SELECT count(*) "
                    + "FROM "
                    + "TagDraft NATURAL JOIN NormalizedTag "
                    + "WHERE "
                    + "tagDraftFacetId IS NOT NULL "
                    + "AND "
                    + "tagDraftFacetId <> 0 "
                    + "AND "
                    + "normalizedTagFacetId <> 5";

            PreparedStatement ps = conn.prepareStatement(sqlQuery);

            ResultSet rs = ps.executeQuery();

            int total=0;

            if(rs.next()){
                total = rs.getInt(1);
            }

            sqlQuery = ""
                    + "SELECT count(*) "
                    + "FROM "
                    + "TagDraft NATURAL JOIN NormalizedTag "
                    + "WHERE "
                    + "tagDraftFacetId IS NOT NULL "
                    + "AND "
                    + "tagDraftFacetId <> 0 "
                    + "AND "
                    + "normalizedTagFacetId <> 5 "
                    + "AND "
                    + "tagDraftFacetId = normalizedTagFacetId";

            if (total != 0) {

                PreparedStatement ps2 = conn.prepareStatement(sqlQuery);

                ResultSet rs2 = ps2.executeQuery();

                int right=0;

                if(rs2.next()){
                    right = rs2.getInt(1);
                }

                if (rs2 != null) {
                    rs2.close();
                }
                if (ps2 != null) {
                    ps2.close();
                }

                accuracy = (float) right / total;

            }

            if (rs != null) {
                rs.close();
            }

            if (ps != null) {
                ps.close();
            }

        } catch (Exception ex) {
            Logger.getLogger(MySQLResultsExplorerDAO.class.getName()).log(Level.SEVERE, null, ex);
        }

        return accuracy;

    }

    public Float getNormalizedTagsPercentage() throws SQLException{
        Float result = null;

        try {

            Connection conn = MySQLDAOFactory.createConnection();

            String sqlQuery = ""
                    + "SELECT count(*) "
                    + "FROM "
                    + "TagDraft ";

            PreparedStatement ps = conn.prepareStatement(sqlQuery);

            ResultSet rs = ps.executeQuery();

            int total=0;

            if(rs.next()){
                total = rs.getInt(1);
            }

            sqlQuery = ""
                    + "SELECT count(*) "
                    + "FROM "
                    + "TagDraft "
                    + "WHERE "
                    + "normalizedTagId IS NOT NULL ";

            if (total != 0) {

                PreparedStatement ps2 = conn.prepareStatement(sqlQuery);

                ResultSet rs2 = ps2.executeQuery();

                int right=0;

                if(rs2.next()){
                    right = rs2.getInt(1);
                }

                if (rs2 != null) {
                    rs2.close();
                }
                if (ps2 != null) {
                    ps2.close();
                }

                result = (float) right / total;

            }

            if (rs != null) {
                rs.close();
            }

            if (ps != null) {
                ps.close();
            }

        } catch (Exception ex) {
            Logger.getLogger(MySQLResultsExplorerDAO.class.getName()).log(Level.SEVERE, null, ex);
        }

        return result;
    }

    public List<TagDraft> getResultsxTagDraft() throws SQLException {
        List<TagDraft> tdl = new ArrayList<TagDraft>();

        Connection conn = MySQLDAOFactory.createConnection();

        String SQLquery =  ""
                + "SELECT "
                    + "TagDraft.tagDraftId AS id, "
                    + "TagDraft.tagDraftName AS name, "
                    + "Cathegory.tagDraftId AS cathegoryId, "
                    + "Cathegory.tagDraftName AS cathegoryName, "
                    + "TagDraft.tagDraftFacetId AS facet, "
                    + "TagDraft.normalizedTagId AS normalizedId, "
                    + "normalizedTagName AS normalizedName, "
                    + "normalizedTagFacetId AS predictedFacet "
                + "FROM "
                    + "(TagDraft "
                        + "LEFT OUTER JOIN "
                            + "TagDraft AS Cathegory "
                        + "ON TagDraft.tagDraftCathegoryId = Cathegory.tagDraftId"
                    + ") "
                        + "LEFT OUTER JOIN "
                            + "NormalizedTag "
                        + "ON TagDraft.normalizedTagId = NormalizedTag.normalizedTagId";

        PreparedStatement ps = conn.prepareStatement(SQLquery);

        ResultSet rs = ps.executeQuery();

        while(rs.next()){

            int tdId = rs.getInt(1);
            String tdName = rs.getString(2);
            Integer cathId = rs.getInt(3);
            if(rs.wasNull()){
                cathId = null;
            }
            String cathName = rs.getString(4);
            if(rs.wasNull()){
                cathName = null;
            }
            Integer fId = rs.getInt(5);
            if(rs.wasNull()){
                fId = null;
            }
            Integer ntId = rs.getInt(6);
            if(rs.wasNull()){
                ntId = null;
            }
            String ntName = rs.getString(7);
            if(rs.wasNull()){
                ntName = null;
            }
            Integer pfId = rs.getInt(8);
            if(rs.wasNull()){
                pfId = null;
            }

            TagDraft td = new TagDraft();
            td.setId(tdId);
            td.setName(tdName);
            if(fId!=null){
                td.setFacet(Facet.getFacet(fId));
            }

            if(cathId != null){
                TagDraft cath = new TagDraft();
                cath.setId(cathId);
                cath.setName(cathName);
                td.setCathegory(cath);
            }

            if(ntId != null){
                NormalizedTag nt = new NormalizedTag();
                nt.setId(ntId);
                nt.setName(ntName);
                nt.setFacet(Facet.getFacet(pfId));
                td.setNormalizedTag(nt);
            }

            tdl.add(td);

        }

        if(rs!=null){
            rs.close();
        }

        if(ps!=null){
            ps.close();
        }

        if(conn!=null){
            conn.close();
        }

        return tdl;
    }

}
