/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package newsAnalyzer.dataModel.containers;

import java.util.Comparator;
import java.util.TreeSet;
import newsAnalyzer.dataModel.Article;
import newsAnalyzer.dataModel.Author;
import newsAnalyzer.dataModel.ArticleSource;
import newsAnalyzer.dataModel.NormalizedTag;
import newsAnalyzer.dataModel.TagDraft;

/**
 *
 * @author rcastari
 */
public class SortedListFactory {

    public static final int SORTED_BY_ID = 0;
    public static final int SORTED_BY_NAME = 1;
    
    public static TreeSet<Article> getArticleSortedList(int comparator) {
        TreeSet<Article> list = null;
        switch (comparator) {
            case SORTED_BY_ID:
                list = new TreeSet<Article>(new Comparator<Article>() {

                    public int compare(Article a1, Article a2) {
                        if (a1.getId() == null || a2.getId() == null) {
                            throw new NullPointerException("Error: article Id is null");
                        }
                        return a1.getId().compareTo(a2.getId());
                    }
                });
        }
        return list;
    }

    public static TreeSet<TagDraft> getTagDraftSortedList(int comparator) {
        TreeSet<TagDraft> list = null;
        switch (comparator) {
            case SORTED_BY_ID:
                list = new TreeSet<TagDraft>(new Comparator<TagDraft>() {

                    public int compare(TagDraft t1, TagDraft t2) {
                        if (t1.getId() == null || t2.getId() == null) {
                            throw new NullPointerException("Error: tag Id is null");
                        }
                        return t1.getId().compareTo(t2.getId());
                    }
                });
        }
        return list;
    }

    public static TreeSet<NormalizedTag> getNormalizedTagSortedList(int comparator) {
        TreeSet<NormalizedTag> list = null;
        switch (comparator) {
            case SORTED_BY_ID:
                list = new TreeSet<NormalizedTag>(new Comparator<NormalizedTag>() {

                    public int compare(NormalizedTag t1, NormalizedTag t2) {
                        int result = 0;
                        if (t1.getId() == null && t2.getId() == null) {
                            if(t1.getFacet()!=null){
                                if(t2.getFacet()!=null){
                                    if(t1.getFacet().getId()<t2.getFacet().getId()){
                                        result = -1;
                                    }else if(t1.getFacet().getId()>t2.getFacet().getId()){
                                        result = 1;
                                    }else{
                                        result = t1.getName().compareTo(t2.getName());
                                    }
                                }else{
                                    result = 1;
                                }
                            }else if(t2.getFacet()!=null){
                                result = -1;
                            }else{
                                result = t1.getName().compareTo(t2.getName());
                            }
                        }else if(t1.getId() == null){
                            result = -1;
                        }else if(t2.getId() == null){
                            result = 1;
                        }else{
                            result = t1.getId().compareTo(t2.getId());
                        }
                        return result;
                    }
                });
        }
        return list;
    }

    public static TreeSet<Author> getAuthorSortedList(int comparator) {
        TreeSet<Author> list = null;
        switch (comparator) {
            case SORTED_BY_ID:
                list = new TreeSet<Author>(new Comparator<Author>() {

                    public int compare(Author a1, Author a2) {
                        if (a1.getId() == null || a2.getId() == null) {
                            throw new NullPointerException("Error: author Id is null");
                        }
                        return a1.getId().compareTo(a2.getId());
                    }
                });
        }
        return list;
    }

    public static TreeSet<ArticleSource> getArticleSourceSortedList(int comparator) {
        TreeSet<ArticleSource> list = null;
        switch (comparator) {
            case SORTED_BY_ID:
                list = new TreeSet<ArticleSource>(new Comparator<ArticleSource>() {

                    public int compare(ArticleSource as1, ArticleSource as2) {
                        if (as1.getId() == null || as2.getId() == null) {
                            throw new NullPointerException("Error: ArticleSource Id is null");
                        }
                        return as1.getId().compareTo(as2.getId());
                    }
                });
                break;
            case SORTED_BY_NAME:
                list = new TreeSet<ArticleSource>(new Comparator<ArticleSource>() {

                    public int compare(ArticleSource as1, ArticleSource as2) {
                        if (as1.getName() == null || as2.getName() == null) {
                            throw new NullPointerException("Error: ArticleSource Name is null");
                        }
                        return as1.getId().compareTo(as2.getId());
                    }
                });
                break;
        }
        return list;
    }

}
