/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package newsAnalyzer.retrievers;

import java.lang.reflect.Method;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import newsAnalyzer.NewsAnalyzerProperties;
import newsAnalyzer.dataModel.Article;
import newsAnalyzer.dataModel.ArticleSource;
import newsAnalyzer.dataModel.DAOs.DAOFactory;
import newsAnalyzer.retrievers.gui.ArticleRetrieverGUI;

/**
 *
 * @author rcastari
 */
public class ArticleRetrieverModule extends Thread {

    private HashMap<Integer,ArticleRetriever> armap = new HashMap<Integer,ArticleRetriever>();
    private HashMap<Integer,ArticleSource> asmap = new HashMap<Integer,ArticleSource>();

    private long updateTime = 0;
    private Date sinceDate = null;
    private List<ArticleSource> asl = new ArrayList<ArticleSource>();
    private ArticleRetrieverGUI argui = null;
    private int action;
    private static final int AUTOMATIC_RETRIEVE = 0;
    private static final int IMMEDIATE_RETRIEVE = 1;

    private boolean stop = false;
    private boolean end = false;

    //TODO Obtenir els mòduls instal·lats
    public ArticleRetrieverModule() throws Exception {

        List<String> installedModulesPropertiesClass = ArticleRetrieverModuleProperties.getInstance().getArticleRetrieverPropertiesClassList();
        Iterator<String> propertiesClassIterator = installedModulesPropertiesClass.iterator();
        while(propertiesClassIterator.hasNext()){
            ArticleRetriever ar = null;
            ArticleRetrieverProperties arp = null;

            String classToLoad = "newsAnalyzer.retrievers."+propertiesClassIterator.next();

            try {

                Class c = Class.forName(classToLoad);
                Method method = c.getMethod("getInstance", new Class[0]);
                arp = (ArticleRetrieverProperties) method.invoke(null, new Object[0]);

                classToLoad = "newsAnalyzer.retrievers."+arp.getMainClass();

                c = Class.forName(classToLoad);
                method = c.getMethod("getInstance", new Class[0]);
                ar = (ArticleRetriever) method.invoke(null, new Object[0]);

            } catch (IllegalAccessException ex) {
                Logger.getLogger(ArticleRetrieverModule.class.getName()).log(Level.SEVERE, "Unable to load "+classToLoad, ex);
                throw ex;
            } catch (ClassNotFoundException ex) {
                Logger.getLogger(ArticleRetrieverModule.class.getName()).log(Level.SEVERE, "Unable to load "+classToLoad, ex);
                throw ex;
            } catch (ClassCastException ex) {
                Logger.getLogger(ArticleRetrieverModule.class.getName()).log(Level.SEVERE, "Unable to load "+classToLoad, ex);
                throw ex;
            }

            if(ar!=null){
                armap.put(ar.getArticleSource().getId(),ar);
                asmap.put(ar.getArticleSource().getId(),ar.getArticleSource());
            }
        }


    }
    
    //TODO Mostar-los a la GUI
    public ArticleSource[] getArticleSources(){
        return asmap.values().toArray(new ArticleSource[0]);
    }

    //TODO Executar les ordres provinents de la GUI cap al mòdul corresponent

    public synchronized void retrieveSince(Date pSinceDate, List<ArticleSource> articleSourceList, ArticleRetrieverGUI argui) throws Exception {
        this.action = IMMEDIATE_RETRIEVE;
        this.sinceDate = pSinceDate;
        this.asl = articleSourceList;
        this.argui = argui;
        if(!this.isAlive()){
            this.start();
        }else{
            this.notifyAll();
        }
    }

    private void retrieveSince(Date pSinceDate, List<ArticleSource> articleSourceList) throws Exception {

        Calendar cSince = Calendar.getInstance();
        cSince.setTime(pSinceDate);

        Calendar cNow = Calendar.getInstance();
        cNow.setTime(new Date());

        if (NewsAnalyzerProperties.getInstance().debug()) {
            System.out.println("Retrieving articles since date: " + (new SimpleDateFormat("dd-MM-yyyy")).format(cSince.getTime()));
        }

        while (cSince.before(cNow) && !stop) {

            if (!(cSince.get(Calendar.YEAR) == cNow.get(Calendar.YEAR)
                    && cSince.get(Calendar.MONTH) == cNow.get(Calendar.MONTH)
                    && cSince.get(Calendar.DATE) == cNow.get(Calendar.DATE))) {

                sinceDate = cSince.getTime();

                Iterator<ArticleSource> asi = articleSourceList.iterator();

                while (asi.hasNext()) {

                    ArticleRetriever articleRetriever = this.armap.get(asi.next().getId());

                    List<Article> articlesFrom = articleRetriever.retrieveFrom(sinceDate);

                    Iterator<Article> ai = articlesFrom.iterator();

                    while (ai.hasNext() && !stop) {
                        DAOFactory.getDAOFactory().getArticleDAO().load(ai.next());
                    }

                }

            }

            cSince.add(Calendar.DATE, 1);
        }
    }
    
    public synchronized void setStartInitValues(){
        stop = false;
    }

    public synchronized void retrieveSince(Date pSinceDate, List<ArticleSource> pArticleSourceList, long pUpdateTime) throws Exception {

        action = AUTOMATIC_RETRIEVE;

        this.setStartInitValues();

        sinceDate = pSinceDate;
        asl = pArticleSourceList;
        updateTime = pUpdateTime;

        if(!this.isAlive()){
            this.start();
        }else{
            this.notifyAll();
        }

    }

    public synchronized void invokeEnd() {
        invokeStop();
        end = true;
        this.notifyAll();
    }
    
    public synchronized void invokeStop() {
        stop = true;
    }
    
    @Override
    public void run() {
        while (!end) {
            switch(action){
                case AUTOMATIC_RETRIEVE:
                    while (!stop) {
                        try {
                            this.retrieveSince(sinceDate, asl);
                            this.invokeWait(updateTime);
                        } catch (InterruptedException ex) {
                            Logger.getLogger(ArticleRetrieverModule.class.getName()).log(Level.SEVERE, null, ex);
                        } catch (Exception ex) {
                            Logger.getLogger(ArticleRetrieverModule.class.getName()).log(Level.SEVERE, null, ex);
                        }
                    }
                    break;
                case IMMEDIATE_RETRIEVE:
                    try {
                        this.retrieveSince(sinceDate, asl);
                    } catch (Exception ex) {
                        Logger.getLogger(ArticleRetrieverModule.class.getName()).log(Level.SEVERE, null, ex);
                        //TODO avisar a la GUI de l'error
                    }
                    argui.stopRetrieving();
                    break;
            }
            if (!end) {
                try {
                    this.invokeWait();
                } catch (InterruptedException ex) {
                    Logger.getLogger(ArticleRetrieverModule.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }
    }

    public synchronized void invokeWait() throws InterruptedException{
        this.wait();
    }

    public synchronized void invokeWait(long milliseconds) throws InterruptedException{
        this.wait(milliseconds);
    }

}