/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package newsAnalyzer.retrievers.guardian;

import java.sql.SQLException;
import newsAnalyzer.dataModel.Article;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import newsAnalyzer.NewsAnalyzerProperties;
import newsAnalyzer.dataModel.Author;
import newsAnalyzer.dataModel.DAOs.DAOFactory;
import newsAnalyzer.dataModel.ArticleSource;
import newsAnalyzer.dataModel.TagDraft;
import newsAnalyzer.retrievers.ArticleRetriever;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;

/**
 *
 * @author rcastari
 */
public class TheGuardianArticleRetriever implements ArticleRetriever {

    String baseURL = "http://content.guardianapis.com/search";
    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
    private int offset;
    private int nextId;
    private Date date;
    private static TheGuardianArticleRetriever articleRetriever = null;
    private ArticleSource articleSource;
    private HashMap<TagDraft, TagDraft> tagsMap = new HashMap<TagDraft, TagDraft>();

    private TheGuardianArticleRetriever() throws Exception {

        //sdf.setTimeZone(TimeZone.getTimeZone("Europe/London"));
        articleSource = new ArticleSource();
        articleSource.setName(TheGuardianRetrieverProperties.getInstance().getProperty(TheGuardianRetrieverProperties.ARTICLE_SOURCE_NAME));
        try {
            articleSource = DAOFactory.getDAOFactory().getArticleSourceDAO().load(articleSource);
        } catch (SQLException ex) {
            Logger.getLogger(TheGuardianArticleRetriever.class.getName()).log(Level.SEVERE, null, ex);
            //TODO Customize exception
            throw new Exception("");
        }

    }

    public ArticleSource getArticleSource() {
        return articleSource;
    }

    public static TheGuardianArticleRetriever getInstance() throws Exception {
        if (articleRetriever == null) {
            articleRetriever = new TheGuardianArticleRetriever();
        }
        return articleRetriever;
    }

    public List<Article> retrieveFrom(Date date) throws Exception {

        List<Article> list = new ArrayList<Article>();

        this.date = date;

        nextId = DAOFactory.getDAOFactory().getArticleDAO().getArticleCount(date, articleSource);

        if (NewsAnalyzerProperties.getInstance().debug()) {

            /*
            SimpleDateFormat londonSDF = new SimpleDateFormat("yyyy-MM-dd");
            londonSDF.setTimeZone(TimeZone.getTimeZone("Europe/London"));



            SimpleDateFormat ourTimeZoneSDF = new SimpleDateFormat("yyyy-MM-dd");
             */

            System.out.println("Article Source: " + articleSource);

            //System.out.println("Retrieving articles from local London date: " + sdf.format(date));
            System.out.println("Retrieving articles from date: " + sdf.format(date));
            //System.out.println(" Local date in our Time Zone (" + ourTimeZoneSDF.getTimeZone().getID() + "): " + ourTimeZoneSDF.format(date));
            System.out.println("Next article for this date: " + nextId);

        }

        int number_of_articles_retrieved = 0;

        boolean end = false;

        BufferedReader in = null;

        offset = nextId / 10;

        Long total = getTotalFromDate(date);

        if ((10 + (offset + 1) * 10) >= total) {
            end = true;
        }

        try {

            while (!end) {

                String request = baseURL + "?tag=type%2Farticle%2Ctone%2Fnews"
                        + "&from-date="
                        + sdf.format(date)
                        + "&to-date="
                        + sdf.format(date)
                        + "&format=json&show-tags=keyword%2Ccontributor"
                        + "&page=" + (offset + 1)
                        + "&order-by=oldest";

                URL url = new URL(request);
                HttpURLConnection httpConn = (HttpURLConnection) url.openConnection();

                if (NewsAnalyzerProperties.getInstance().debug()) {
                    System.out.println(httpConn);
                }

                boolean success = false;

                //TODO: Revisar - En cas d'error repeteix la sol.licitud HTTP fins a 5 vegades

                int n = 0;

                while (!success && n < 5) {
                    try {
                        in = new BufferedReader(new InputStreamReader(httpConn.getInputStream()));
                        success = true;
                    } catch (IOException ex) {
                        //TODO tractar excepció, aturar l'actualització de la bbdd?
                        String msg = "";
                        String aux;
                        BufferedReader msgr = new BufferedReader(new InputStreamReader(httpConn.getErrorStream()));
                        while ((aux = msgr.readLine()) != null) {
                            msg += aux;
                        }
                        Logger.getLogger(TheGuardianArticleRetriever.class.getName()).log(Level.SEVERE, msg, ex);
                        n++;
                    }
                }

                if (!success) {
                    //TODO customize exception
                    throw new Exception("Unable to access the The Guardian API or Malformed Query");
                }

                String response = "";

                String inputLine;

                while ((inputLine = in.readLine()) != null) {

                    response += inputLine + System.getProperty("line.separator");

                }

                in.close();
                httpConn.disconnect();

                Object o_json = null;

                JSONParser parser = new JSONParser();

                o_json = parser.parse(response);

                if (o_json instanceof JSONObject) {
                    JSONObject jso = (JSONObject) o_json;



                    if (jso.containsKey("response")) {
                        Object o_response = jso.get("response");
                        if (o_response instanceof JSONObject) {
                            JSONObject jso_response = (JSONObject) o_response;

                            List<Article> l2 = new ArrayList<Article>();

                            if (jso_response.containsKey("results")) {
                                Object o_results = jso_response.get("results");
                                if (o_results instanceof JSONArray) {
                                    JSONArray jsa_results = (JSONArray) o_results;
                                    l2.addAll(this.parseArticleList(jsa_results));
                                } else {
                                    throw new Exception();
                                }
                            }

                            number_of_articles_retrieved += l2.size();

                            list.addAll(l2);


                        } else {
                            throw new Exception();
                        }
                    } else {
                        throw new Exception();
                    }

                    if ((10 + (offset + 1) * 10) >= total) {
                        end = true;
                    } else {
                        offset++;
                    }

                } else {
                    throw new Exception();
                }

            }

        } catch (NumberFormatException ex) {
            Logger.getLogger(TheGuardianArticleRetriever.class.getName()).log(Level.SEVERE, null, ex);
        } catch (ParseException ex) {
            Logger.getLogger(TheGuardianArticleRetriever.class.getName()).log(Level.SEVERE, null, ex);
        } catch (MalformedURLException ex) {
            Logger.getLogger(TheGuardianArticleRetriever.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(TheGuardianArticleRetriever.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException ex) {
                    Logger.getLogger(TheGuardianArticleRetriever.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }

        if (NewsAnalyzerProperties.getInstance().debug()) {
            System.out.println("Articles retrieved for this date = " + number_of_articles_retrieved);
        }

        return list;
    }

    private List<Article> parseArticleList(JSONArray jsa) throws Exception {

        List<Article> l = new ArrayList<Article>();

        int n = 0;
        Iterator<Object> i = jsa.iterator();

        while (i.hasNext() && nextId > (n + 10 * offset)) {
            i.next();
            n++;
        }

        while (i.hasNext()) {
            Object jsa_o = i.next();
            if (jsa_o instanceof JSONObject) {
                JSONObject jsa_jso = (JSONObject) jsa_o;
                Article a = new Article();
                //todo adaptar a la data de ny
                a.setDate(date);
                a.setSource(articleSource);
                if (jsa_jso.get("webTitle") instanceof String) {
                    a.setTitle((String) jsa_jso.get("webTitle"));
                } else {
                    throw new Exception();
                }
                if (jsa_jso.containsKey("byline")) {
                    Object a_o = jsa_jso.get("byline");
                    if (a_o instanceof String) {
                        String byLine = (String) a_o;
                        if (byLine.startsWith("By ")) {
                            byLine = byLine.substring(3);
                        }
                        String authorNames[] = byLine.split(" and |;|,");
                        for (int p = 0; p < authorNames.length; p++) {
                            Author author = new Author();
                            author.setName(authorNames[p]);
                            a.addAuthor(author);
                        }
                    } else {
                        throw new Exception();
                    }
                }
                if (jsa_jso.containsKey("webUrl")) {
                    Object a_o = jsa_jso.get("webUrl");
                    if (a_o instanceof String) {
                        a.setUrl((String) a_o);
                    } else {
                        throw new Exception();
                    }
                }
                if (jsa_jso.containsKey("tags")) {
                    Object a_o = jsa_jso.get("tags");
                    if (a_o instanceof JSONArray) {
                        JSONArray jsa_tags = (JSONArray) a_o;

                        Iterator<Object> i_tag = jsa_tags.iterator();

                        while (i_tag.hasNext()) {
                            JSONObject jso_o = (JSONObject) i_tag.next();
                            String type = (String) jso_o.get("type");
                            if (type.equalsIgnoreCase("keyword")) {
                                TagDraft td = new TagDraft();
                                td.setName((String) jso_o.get("webTitle"));
                                if (jso_o.containsKey("sectionId")) {
                                    String cathegoryName = (String) jso_o.get("sectionId");
                                    TagDraft tdCathegorySearchParameters = new TagDraft();
                                    tdCathegorySearchParameters.setName(cathegoryName);
                                    TagDraft tdCathegory = tagsMap.get(tdCathegorySearchParameters);
                                    if (tdCathegory == null) {
                                        tdCathegory = tdCathegorySearchParameters;
                                        tagsMap.put(tdCathegory, tdCathegory);
                                    }
                                    td.setCathegory(tdCathegory);
                                }
                                a.addTagDraft(td);
                                if (td.getCathegory() != null) {
                                    if (!a.getTagDraftsList().contains(td.getCathegory())) {
                                        a.addTagDraft(td.getCathegory());
                                    }
                                }
                            } else if (type.equalsIgnoreCase("contributor")) {
                                Author author = new Author();
                                author.setName((String) jso_o.get("webTitle"));
                                a.addAuthor(author);
                            }
                        }

                    } else {
                        throw new Exception();
                    }
                }
                l.add(a);
            } else {
                throw new Exception();
            }
            nextId++;
        }

        return l;
    }

    public static void main(String args[]) {
        try {
            TheGuardianArticleRetriever ar = new TheGuardianArticleRetriever();
            List<Article> list = ar.retrieveFrom((new SimpleDateFormat("yyyyMMdd hh:mm:ss")).parse("20110413 00:50:18"));
            Iterator<Article> i = list.iterator();
            while (i.hasNext()) {
                System.out.println(i.next());
            }
        } catch (Exception ex) {
            Logger.getLogger(TheGuardianArticleRetriever.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    private long getTotalFromDate(Date date) throws Exception {
        long total = 0;

        String request = baseURL + "?tag=type%2Farticle%2Ctone%2Fnews"
                + "&from-date="
                + sdf.format(date)
                + "&to-date="
                + sdf.format(date)
                + "&format=json"
                + "&page=" + 1
                + "&order-by=oldest";

        URL url = new URL(request);
        HttpURLConnection httpConn = (HttpURLConnection) url.openConnection();

        if (NewsAnalyzerProperties.getInstance().debug()) {
            System.out.println(httpConn);
        }

        boolean success = false;

        //TODO: Revisar - En cas d'error repeteix la sol.licitud HTTP fins a 5 vegades

        boolean end = false;

        BufferedReader in = null;

        int n = 0;

        while (!success && n < 5) {
            try {
                in = new BufferedReader(new InputStreamReader(httpConn.getInputStream()));
                success = true;
            } catch (IOException ex) {
                //TODO tractar excepció, aturar l'actualització de la bbdd?
                String msg = "";
                String aux;
                BufferedReader msgr = new BufferedReader(new InputStreamReader(httpConn.getErrorStream()));
                while ((aux = msgr.readLine()) != null) {
                    msg += aux;
                }
                Logger.getLogger(TheGuardianArticleRetriever.class.getName()).log(Level.SEVERE, msg, ex);
                n++;
            }
        }

        if (!success) {
            //TODO customize exception
            throw new Exception("Unable to access the The Guardian API or Malformed Query");
        }

        String response = "";

        String inputLine;

        while ((inputLine = in.readLine()) != null) {

            response += inputLine + System.getProperty("line.separator");

        }

        in.close();
        httpConn.disconnect();

        Object o_json = null;

        JSONParser parser = new JSONParser();

        o_json = parser.parse(response);

        if (o_json instanceof JSONObject) {
            JSONObject jso = (JSONObject) o_json;



            if (jso.containsKey("response")) {
                Object o_response = jso.get("response");
                if (o_response instanceof JSONObject) {
                    JSONObject jso_response = (JSONObject) o_response;

                    if (jso_response.containsKey("total")) {
                        Object o_total = jso_response.get("total");
                        if (o_total instanceof Long) {
                            total = (Long) o_total;
                            if (NewsAnalyzerProperties.getInstance().debug()) {
                                System.out.println("Total = " + total);
                            }
                        } else {
                            throw new Exception();
                        }
                    }
                }
            }
        }
        return total;
    }
}
