/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package newsAnalyzer.retrievers.nyt;

import java.sql.SQLException;
import newsAnalyzer.dataModel.Article;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;
import java.util.logging.Level;
import java.util.logging.Logger;
import newsAnalyzer.NewsAnalyzerProperties;
import newsAnalyzer.dataModel.Author;
import newsAnalyzer.dataModel.DAOs.DAOFactory;
import newsAnalyzer.dataModel.Facet;
import newsAnalyzer.dataModel.ArticleSource;
import newsAnalyzer.dataModel.TagDraft;
import newsAnalyzer.retrievers.ArticleRetriever;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;

/**
 *
 * @author rcastari
 */


public class NYTArticleRetriever implements ArticleRetriever {

    private String apiKey;
    String baseURL = "http://api.nytimes.com/svc/search/v1/article";
    SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
    private int offset;
    private int nextId;
    private Date date;
    private static NYTArticleRetriever articleRetriever = null;
    private ArticleSource articleSource;
    private HashMap<TagDraft,TagDraft> tagsMap = new HashMap<TagDraft,TagDraft>();

    private NYTArticleRetriever() throws Exception {

        apiKey = NYTRetrieverProperties.getInstance().getProperty(NYTRetrieverProperties.NYT_API_KEY);

        //sdf.setTimeZone(TimeZone.getTimeZone("America/New_York"));

        articleSource = new ArticleSource();
        articleSource.setName(NYTRetrieverProperties.getInstance().getProperty(NYTRetrieverProperties.ARTICLE_SOURCE_NAME));
        try {
            articleSource = DAOFactory.getDAOFactory().getArticleSourceDAO().load(articleSource);
        } catch (SQLException ex) {
            Logger.getLogger(NYTArticleRetriever.class.getName()).log(Level.SEVERE, null, ex);
            //TODO Customize exception
            throw new Exception("");
        }

    }

    public ArticleSource getArticleSource(){
        return articleSource;
    }

    public static NYTArticleRetriever getInstance() throws Exception {
        if (articleRetriever == null) {
            articleRetriever = new NYTArticleRetriever();
        }
        return articleRetriever;
    }

    public List<Article> retrieveFrom(Date date) throws Exception {

        List<Article> list = new ArrayList<Article>();

        this.date = date;

        nextId = DAOFactory.getDAOFactory().getArticleDAO().getArticleCount(date, articleSource);

        if (NewsAnalyzerProperties.getInstance().debug()) {

            /*
            SimpleDateFormat londonSDF = new SimpleDateFormat("yyyy-MM-dd");
            londonSDF.setTimeZone(TimeZone.getTimeZone("America/New_York"));

            SimpleDateFormat ourTimeZoneSDF = new SimpleDateFormat("yyyy-MM-dd");
             *
             */

            System.out.println("Article Source: "+articleSource);

            //System.out.println("Retrieving articles from local NY date: " + sdf.format(date));
            System.out.println("Retrieving articles from date: " + sdf.format(date));
            //System.out.println(" Local date in our Time Zone (" + ourTimeZoneSDF.getTimeZone().getID() + "): " + ourTimeZoneSDF.format(date));
            System.out.println("Next article for this date: " + nextId);

        }

        int number_of_articles_retrieved = 0;

        boolean end = false;

        BufferedReader in = null;

        offset = nextId / 10;

        Long total = this.getTotalFromDate(date);

        if ((10 + offset * 10) >= total) {
            end = true;
        }

        try {

            while (!end) {

                String request = baseURL + "?format=json&query=date%3A%5B" + sdf.format(date) + "%5D+classifiers_facet%3A%5BTop%2FNews%5D"
                        + "&fields=title%2Curl%2Cbyline%2Cclassifiers_facet%2Cdes_facet%2Cgeo_facet%2Cper_facet%2Corg_facet"
                        + "&offset=" + offset
                        + "&api-key=" + apiKey;

                URL url = new URL(request);
                HttpURLConnection httpConn = (HttpURLConnection)url.openConnection();

                if(NewsAnalyzerProperties.getInstance().debug()){
                    System.out.println(httpConn);
                }

                boolean success = false;

                //TODO: Revisar - En cas d'error repeteix la sol.licitud HTTP fins a 5 vegades

                int n = 0;

                while (!success && n < 5) {
                    try {
                        in = new BufferedReader(new InputStreamReader(httpConn.getInputStream()));
                        success = true;
                    } catch (IOException ex) {
                        //TODO tractar excepció, aturar l'actualització de la bbdd?
                        String msg = "";
                        String aux;
                        BufferedReader msgr = new BufferedReader(new InputStreamReader(httpConn.getErrorStream()));
                        while ((aux = msgr.readLine()) != null) {
                            msg += aux;
                        }
                        Logger.getLogger(NYTArticleRetriever.class.getName()).log(Level.SEVERE, null, ex);
                        n++;
                    }
                }

                if(!success){
                    //TODO customize exception
                    throw new Exception("Unable to access the NYT API or Malformed Query");
                }

                String response = "";

                String inputLine;

                while ((inputLine = in.readLine()) != null) {

                    response += inputLine + System.getProperty("line.separator");

                }

                in.close();
                httpConn.disconnect();

                Object o_response = null;

                JSONParser parser = new JSONParser();

                o_response = parser.parse(response);

                if (o_response instanceof JSONObject) {
                    JSONObject jso = (JSONObject) o_response;

                    List<Article> l2 = this.parseArticleList(jso);

                    number_of_articles_retrieved += l2.size();

                    list.addAll(l2);

                    if ((10 + offset * 10) >= total) {
                        end = true;
                    } else {
                        offset++;
                    }

                } else {
                    throw new Exception();
                }

            }

        } catch (NumberFormatException ex) {
            Logger.getLogger(NYTArticleRetriever.class.getName()).log(Level.SEVERE, null, ex);
        } catch (ParseException ex) {
            Logger.getLogger(NYTArticleRetriever.class.getName()).log(Level.SEVERE, null, ex);
        } catch (MalformedURLException ex) {
            Logger.getLogger(NYTArticleRetriever.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(NYTArticleRetriever.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            if(in!=null){
                try {
                    in.close();
                } catch (IOException ex) {
                    Logger.getLogger(NYTArticleRetriever.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }

        if (NewsAnalyzerProperties.getInstance().debug()) {
            System.out.println("Articles retrieved for this date = " + number_of_articles_retrieved);
        }

        return list;
    }

    private List<Article> parseArticleList(JSONObject jso) throws Exception {

        List<Article> l = new ArrayList<Article>();

        if (jso.containsKey("results")) {
            Object o = jso.get("results");
            if (o instanceof JSONArray) {
                JSONArray jsa = (JSONArray) o;
                int n = 0;
                Iterator<Object> i = jsa.iterator();

                while (i.hasNext() && nextId > (n + 10 * offset)) {
                    i.next();
                    n++;
                }

                while (i.hasNext()) {
                    Object jsa_o = i.next();
                    if (jsa_o instanceof JSONObject) {
                        JSONObject jsa_jso = (JSONObject) jsa_o;
                        Article a = new Article();

                        HashMap<String,List<TagDraft>> tdm = new HashMap<String,List<TagDraft>>();

                        //TODO adaptar a la data de ny
                        a.setDate(date);
                        a.setSource(articleSource);
                        if (jsa_jso.get("title") instanceof String) {
                            a.setTitle((String) jsa_jso.get("title"));
                        } else {
                            throw new Exception();
                        }
                        if (jsa_jso.containsKey("byline")) {
                            Object a_o = jsa_jso.get("byline");
                            if (a_o instanceof String) {
                                String byLine = (String)a_o;
                                if(byLine.startsWith("By ")){
                                    byLine=byLine.substring(3);
                                }
                                String authorNames[] = byLine.split(" and |;|,");
                                for (int p = 0; p < authorNames.length; p++) {
                                    Author author = new Author();
                                    author.setName(authorNames[p]);
                                    a.addAuthor(author);
                                }
                            } else {
                                throw new Exception();
                            }
                        }
                        if (jsa_jso.containsKey("url")) {
                            Object a_o = jsa_jso.get("url");
                            if (a_o instanceof String) {
                                a.setUrl((String) a_o);
                            } else {
                                throw new Exception();
                            }
                        }
                        if (jsa_jso.containsKey("classifiers_facet")) {
                            Object a_o = jsa_jso.get("classifiers_facet");
                            if (a_o instanceof JSONArray) {
                                JSONArray tags = (JSONArray) a_o;

                                tdm=(parseTagsWithCathegory(tags));

                            } else {
                                throw new Exception();
                            }
                        }
                        if (jsa_jso.containsKey("des_facet")) {
                            Object a_o = jsa_jso.get("des_facet");
                            if (a_o instanceof JSONArray) {
                                JSONArray tags = (JSONArray) a_o;
                                tdm=parseTags(tags, Facet.DESCRIPTIVE_SUBJECT, tdm);
                            } else {
                                throw new Exception();
                            }
                        }
                        if (jsa_jso.containsKey("geo_facet")) {
                            Object a_o = jsa_jso.get("geo_facet");
                            if (a_o instanceof JSONArray) {
                                JSONArray tags = (JSONArray) a_o;
                                tdm=parseTags(tags, Facet.GEOGRAPHIC_LOCATION, tdm);
                            } else {
                                throw new Exception();
                            }
                        }
                        if (jsa_jso.containsKey("org_facet")) {
                            Object a_o = jsa_jso.get("org_facet");
                            if (a_o instanceof JSONArray) {
                                JSONArray tags = (JSONArray) a_o;
                                tdm=parseTags(tags, Facet.ORGANIZATION, tdm);
                            } else {
                                throw new Exception();
                            }
                        }
                        if (jsa_jso.containsKey("per_facet")) {
                            Object a_o = jsa_jso.get("per_facet");
                            if (a_o instanceof JSONArray) {
                                JSONArray tags = (JSONArray) a_o;
                                tdm=parseTags(tags, Facet.PERSON, tdm);
                            } else {
                                throw new Exception();
                            }
                        }

                        List<TagDraft> tdl = new ArrayList<TagDraft>();
                        Iterator<String> keySetIterator = tdm.keySet().iterator();
                        while(keySetIterator.hasNext()){
                            tdl.addAll(tdm.get(keySetIterator.next()));
                        }

                        a.addTagDraftList(tdl);
                        l.add(a);
                    } else {
                        throw new Exception();
                    }
                    nextId++;
                }
            } else {
                throw new Exception();
            }
        } else {
            throw new Exception();
        }

        return l;
    }

    private HashMap<String,List<TagDraft>> parseTags(JSONArray jsa, Facet facet, HashMap<String,List<TagDraft>> existingTags) throws Exception {
        if(existingTags==null){
            existingTags = new HashMap<String,List<TagDraft>>();
        }

        Iterator<Object> i = jsa.iterator();

        while (i.hasNext()) {
            Object jsa_o = i.next();
            if (jsa_o instanceof String) {

                String tagName = (String) jsa_o;

                List<TagDraft> tdl = existingTags.get(tagName);

                if(tdl==null){
                    tdl = new ArrayList<TagDraft>();
                    existingTags.put(tagName, tdl);
                }

                Iterator<TagDraft> tdi = tdl.iterator();

                List<TagDraft> candidates = new ArrayList<TagDraft>();

                while(tdi.hasNext()){
                    TagDraft td = tdi.next();
                    if(td.getName().equalsIgnoreCase(tagName)){
                        candidates.add(td);
                    }
                }

                if(candidates.size()==1){
                    if(candidates.get(0).getFacet()==null){
                        candidates.get(0).setFacet(facet);
                    }else if(candidates.get(0).getFacet()==Facet.NULL || candidates.get(0).getFacet()==Facet.DESCRIPTIVE_SUBJECT){
                        candidates.get(0).setFacet(facet);
                    }else{
                        //TODO need to disambiguate
                        candidates.get(0).setFacet(null);
                    }
                }else if(candidates.isEmpty()){
                    TagDraft tag = new TagDraft();
                    tag.setName(tagName);
                    tag.setFacet(facet);
                    tdl.add(tag);
                }else{
                    //TODO need to disambiguate
                }

            } else {
                throw new Exception();
            }
        }
        return existingTags;
    }

    private HashMap<String,List<TagDraft>> parseTagsWithCathegory(JSONArray jsa) throws Exception {
        HashMap<String,List<TagDraft>> tags = new HashMap<String,List<TagDraft>>();

        Iterator<Object> i = jsa.iterator();

        while (i.hasNext()) {
            Object jsa_o = i.next();
            if (jsa_o instanceof String) {

                String tagName = (String) jsa_o;

                if (tagName.startsWith("Top/News/")) {

                    tagName = tagName.substring(9);

                    String[] tagsSplit = tagName.split("/");
                    Queue<String> cathegories = new LinkedList<String>();

                    for(int t = (tagsSplit.length - 1); t >= 0; t--){
                        cathegories.add(tagsSplit[t]);
                    }

                    if(!cathegories.isEmpty()){
                        TagDraft tag = new TagDraft();
                        tag.setName(cathegories.remove());
                        tag.setCathegory(parseCathegories(cathegories));

                        //Vincular els tags obtinguts amb tagsMap, el llistat de tots els tags
                        TagDraft inMemoryTd = tagsMap.get(tag);
                        if(inMemoryTd!=null){
                            tag = inMemoryTd;
                        }

                        //Desar aquests tags al llistat de tags de l'article
                        assignTagDraftHierarchy(tag, tags);
                        
                    }
                }

            } else {
                throw new Exception();
            }
        }
        return tags;
    }

    private TagDraft parseCathegories(Queue<String> cathegories){
        TagDraft cathegory = null;
        if(!cathegories.isEmpty()){
            cathegory = new TagDraft();
            cathegory.setName(cathegories.remove());
            if(!cathegories.isEmpty()){
                cathegory.setCathegory(parseCathegories(cathegories));
            }
            TagDraft inMemoryTd = tagsMap.get(cathegory);
            if(inMemoryTd!=null){
                cathegory=inMemoryTd;
            }
        }
        return cathegory;
    }

    private void assignTagDraftHierarchy(TagDraft td, HashMap<String,List<TagDraft>> assignedTags){
        List<TagDraft> list = assignedTags.get(td.getName());

        if (list == null) {
            list = new ArrayList<TagDraft>();
            assignedTags.put(td.getName(), list);
        }

        if (!list.contains(td)) {
            if(td.getCathegory()!=null){
                assignTagDraftHierarchy(td.getCathegory(), assignedTags);
            }
            list.add(td);
        }
    }

    public static void main(String args[]){
        try {
            NYTArticleRetriever ar = new NYTArticleRetriever();
            List<Article> list = ar.retrieveFrom((new SimpleDateFormat("yyyy-MM-dd hh:mm:ss")).parse("2011-04-13 02:00:00"));
            Iterator<Article> i = list.iterator();
            while(i.hasNext()){
                System.out.println(i.next());
            }
        } catch (Exception ex) {
            Logger.getLogger(NYTArticleRetriever.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    private long getTotalFromDate(Date date) throws Exception {

        long total = 0;

        String request = baseURL + "?format=json&query=date%3A%5B" + sdf.format(date) + "%5D+classifiers_facet%3A%5BTop%2FNews%5D"
                + "&fields=title%2Curl%2Cbyline%2Cclassifiers_facet%2Cdes_facet%2Cgeo_facet%2Cper_facet%2Corg_facet"
                + "&offset=" + 0
                + "&api-key=" + apiKey;

        URL url = new URL(request);
        HttpURLConnection httpConn = (HttpURLConnection) url.openConnection();

        if (NewsAnalyzerProperties.getInstance().debug()) {
            System.out.println(httpConn);
        }

        boolean success = false;

        BufferedReader in = null;

        //TODO: Revisar - En cas d'error repeteix la sol.licitud HTTP fins a 5 vegades

        int n = 0;

        while (!success && n < 5) {
            try {
                in = new BufferedReader(new InputStreamReader(httpConn.getInputStream()));
                success = true;
            } catch (IOException ex) {
                //TODO tractar excepció, aturar l'actualització de la bbdd?
                String msg = "";
                String aux;
                BufferedReader msgr = new BufferedReader(new InputStreamReader(httpConn.getErrorStream()));
                while ((aux = msgr.readLine()) != null) {
                    msg += aux;
                }
                Logger.getLogger(NYTArticleRetriever.class.getName()).log(Level.SEVERE, msg, ex);
                n++;
            }
        }

        if (!success) {
            //TODO customize exception
            throw new Exception("Unable to access the NYT API or Malformed Query");
        }

        String response = "";

        String inputLine;

        while ((inputLine = in.readLine()) != null) {

            response += inputLine + System.getProperty("line.separator");

        }

        in.close();
        httpConn.disconnect();

        Object o_response = null;

        JSONParser parser = new JSONParser();

        o_response = parser.parse(response);

        if (o_response instanceof JSONObject) {
            JSONObject jso = (JSONObject) o_response;

            if (jso.containsKey("total")) {
                Object o_total = jso.get("total");
                if (o_total instanceof Long) {
                    total = (Long) o_total;
                    if (NewsAnalyzerProperties.getInstance().debug()) {
                        System.out.println("Total = " + total);
                    }
                } else {
                    throw new Exception();
                }

            }

        }

        return total;
    }

}